"""Handling and managing an Agent Studio Experimental Config

Copyright PolyAI Limited
"""

import json
import os
from dataclasses import dataclass, field

import jsonschema

import poly.resources.resource_utils as utils
from poly.handlers.protobuf.experimental_config_pb2 import ExperimentalConfig_UpdateConfig
from poly.resources.resource import Resource


@dataclass
class ExperimentalConfig(Resource):
    """ExperimentalConfig resource"""

    config: dict = field(default_factory=dict)

    @property
    def file_path(self) -> str:
        return os.path.join("agent_settings", "experimental_config.json")

    @property
    def raw(self) -> str:
        return json.dumps(self.config, indent=4, sort_keys=True)

    @staticmethod
    def format_resource(content: str, file_name: str = None, **kwargs) -> str:
        """Format the resource content using in-process JSON formatting."""
        return utils.format_json_python(content)

    @staticmethod
    def make_pretty(contents: str, **kwargs) -> str:
        return contents

    @staticmethod
    def from_pretty(contents: str, **kwargs) -> str:
        return contents

    @classmethod
    def read_local_resource(
        cls, file_path: str, resource_id: str, resource_name: str, **kwargs
    ) -> "ExperimentalConfig":
        content = cls.read_to_raw(file_path, **kwargs)
        content_json = json.loads(content)
        return cls(resource_id=resource_id, name=resource_name, config=content_json)

    def build_update_proto(self) -> ExperimentalConfig_UpdateConfig:
        return ExperimentalConfig_UpdateConfig(
            id=self.resource_id,
            features=self.config,
        )

    def build_delete_proto(self):
        return NotImplementedError("ExperimentalConfig does not support deletion.")

    def build_create_proto(self):
        return NotImplementedError("ExperimentalConfig does not support creation.")

    def validate(self, **kwargs):
        # Validate against schema
        openapi_schema = utils.load_yaml(
            open(
                os.path.join(os.path.dirname(__file__), "experimental_config_schema.yaml"),
                encoding="utf-8",
            )
        )
        additional_features = openapi_schema["components"]["schemas"]["additional_features"]
        resolver = jsonschema.RefResolver.from_schema(openapi_schema)

        validator = jsonschema.Draft202012Validator(additional_features, resolver=resolver)
        validator.validate(self.config)

    @property
    def command_type(self) -> str:
        return "experimental_config"

    @property
    def update_command_type(self) -> str:
        return "experimental_config_update_config"

    @staticmethod
    def discover_resources(base_path: str) -> list[str]:
        file_path = os.path.join(base_path, "agent_settings", "experimental_config.json")

        if not os.path.exists(file_path):
            return []

        return [file_path]



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Integrations(_message.Message):
    __slots__ = ("paragon_providers_by_name",)
    class ParagonProvidersByNameEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: ParagonProvider
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[ParagonProvider, _Mapping]] = ...) -> None: ...
    PARAGON_PROVIDERS_BY_NAME_FIELD_NUMBER: _ClassVar[int]
    paragon_providers_by_name: _containers.MessageMap[str, ParagonProvider]
    def __init__(self, paragon_providers_by_name: _Optional[_Mapping[str, ParagonProvider]] = ...) -> None: ...

class ParagonProvider(_message.Message):
    __slots__ = ("provider_id", "available_functions", "timeout", "env_integrations", "integrations_by_id")
    class IntegrationsByIdEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: ParagonIntegration
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[ParagonIntegration, _Mapping]] = ...) -> None: ...
    PROVIDER_ID_FIELD_NUMBER: _ClassVar[int]
    AVAILABLE_FUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    ENV_INTEGRATIONS_FIELD_NUMBER: _ClassVar[int]
    INTEGRATIONS_BY_ID_FIELD_NUMBER: _ClassVar[int]
    provider_id: str
    available_functions: _containers.RepeatedScalarFieldContainer[str]
    timeout: int
    env_integrations: EnvironmentIntegrationRefs
    integrations_by_id: _containers.MessageMap[str, ParagonIntegration]
    def __init__(self, provider_id: _Optional[str] = ..., available_functions: _Optional[_Iterable[str]] = ..., timeout: _Optional[int] = ..., env_integrations: _Optional[_Union[EnvironmentIntegrationRefs, _Mapping]] = ..., integrations_by_id: _Optional[_Mapping[str, ParagonIntegration]] = ...) -> None: ...

class EnvironmentIntegrationRefs(_message.Message):
    __slots__ = ("sandbox", "pre_release", "live")
    SANDBOX_FIELD_NUMBER: _ClassVar[int]
    PRE_RELEASE_FIELD_NUMBER: _ClassVar[int]
    LIVE_FIELD_NUMBER: _ClassVar[int]
    sandbox: str
    pre_release: str
    live: str
    def __init__(self, sandbox: _Optional[str] = ..., pre_release: _Optional[str] = ..., live: _Optional[str] = ...) -> None: ...

class ParagonIntegration(_message.Message):
    __slots__ = ("paragon_connection_id", "paragon_user_id", "metadata", "created_at", "created_by", "updated_at", "updated_by")
    PARAGON_CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    PARAGON_USER_ID_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    UPDATED_BY_FIELD_NUMBER: _ClassVar[int]
    paragon_connection_id: str
    paragon_user_id: str
    metadata: _struct_pb2.Struct
    created_at: _timestamp_pb2.Timestamp
    created_by: str
    updated_at: _timestamp_pb2.Timestamp
    updated_by: str
    def __init__(self, paragon_connection_id: _Optional[str] = ..., paragon_user_id: _Optional[str] = ..., metadata: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., created_by: _Optional[str] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., updated_by: _Optional[str] = ...) -> None: ...

class Integration_Connect(_message.Message):
    __slots__ = ("provider", "paragon_connection_id", "paragon_user_id", "client_env", "metadata")
    PROVIDER_FIELD_NUMBER: _ClassVar[int]
    PARAGON_CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    PARAGON_USER_ID_FIELD_NUMBER: _ClassVar[int]
    CLIENT_ENV_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    provider: str
    paragon_connection_id: str
    paragon_user_id: str
    client_env: str
    metadata: _struct_pb2.Struct
    def __init__(self, provider: _Optional[str] = ..., paragon_connection_id: _Optional[str] = ..., paragon_user_id: _Optional[str] = ..., client_env: _Optional[str] = ..., metadata: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...) -> None: ...

class Integration_Enable(_message.Message):
    __slots__ = ("provider", "paragon_connection_id", "client_env")
    PROVIDER_FIELD_NUMBER: _ClassVar[int]
    PARAGON_CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    CLIENT_ENV_FIELD_NUMBER: _ClassVar[int]
    provider: str
    paragon_connection_id: str
    client_env: str
    def __init__(self, provider: _Optional[str] = ..., paragon_connection_id: _Optional[str] = ..., client_env: _Optional[str] = ...) -> None: ...

class Integration_Disable(_message.Message):
    __slots__ = ("provider", "paragon_connection_id", "client_env")
    PROVIDER_FIELD_NUMBER: _ClassVar[int]
    PARAGON_CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    CLIENT_ENV_FIELD_NUMBER: _ClassVar[int]
    provider: str
    paragon_connection_id: str
    client_env: str
    def __init__(self, provider: _Optional[str] = ..., paragon_connection_id: _Optional[str] = ..., client_env: _Optional[str] = ...) -> None: ...

class Integration_UpdateFunctions(_message.Message):
    __slots__ = ("provider", "available_functions")
    PROVIDER_FIELD_NUMBER: _ClassVar[int]
    AVAILABLE_FUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    provider: str
    available_functions: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, provider: _Optional[str] = ..., available_functions: _Optional[_Iterable[str]] = ...) -> None: ...

class Integration_Disconnect(_message.Message):
    __slots__ = ("provider", "paragon_connection_id")
    PROVIDER_FIELD_NUMBER: _ClassVar[int]
    PARAGON_CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    provider: str
    paragon_connection_id: str
    def __init__(self, provider: _Optional[str] = ..., paragon_connection_id: _Optional[str] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: agent_settings.proto
# Protobuf Python Version: 4.25.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14\x61gent_settings.proto\x12\x05\x61gent\x1a\x1fgoogle/protobuf/timestamp.proto\"\xe5\x02\n\rAgentSettings\x12!\n\x08greeting\x18\x01 \x03(\x0b\x32\x0f.agent.Greeting\x12\'\n\x0bpersonality\x18\x02 \x01(\x0b\x32\x12.agent.Personality\x12\x19\n\x04role\x18\x03 \x01(\x0b\x32\x0b.agent.Role\x12\x34\n\x12\x64isclaimer_message\x18\x04 \x03(\x0b\x32\x18.agent.DisclaimerMessage\x12\x1b\n\x05rules\x18\x05 \x01(\x0b\x32\x0c.agent.Rules\x12G\n\x12language_behaviors\x18\x06 \x03(\x0b\x32+.agent.AgentSettings.LanguageBehaviorsEntry\x1aQ\n\x16LanguageBehaviorsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.agent.LanguageBehavior:\x02\x38\x01\"\x91\x02\n\x08Greeting\x12\x17\n\x0fwelcome_message\x18\x01 \x01(\t\x12\x12\n\ncreated_at\x18\x02 \x01(\t\x12\x12\n\ncreated_by\x18\x03 \x01(\t\x12\x33\n\nupdated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x88\x01\x01\x12\x17\n\nupdated_by\x18\x05 \x01(\tH\x01\x88\x01\x01\x12\x32\n\nreferences\x18\x06 \x01(\x0b\x32\x19.agent.GreetingReferencesH\x02\x88\x01\x01\x12\x15\n\rlanguage_code\x18\x07 \x01(\tB\r\n\x0b_updated_atB\r\n\x0b_updated_byB\r\n\x0b_references\"\xa5\x01\n\x17Greeting_UpdateGreeting\x12\x1c\n\x0fwelcome_message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x32\n\nreferences\x18\x02 \x01(\x0b\x32\x19.agent.GreetingReferencesH\x01\x88\x01\x01\x12\x15\n\rlanguage_code\x18\x03 \x01(\tB\x12\n\x10_welcome_messageB\r\n\x0b_references\"\x83\x01\n\x12GreetingReferences\x12;\n\tvariables\x18\x01 \x03(\x0b\x32(.agent.GreetingReferences.VariablesEntry\x1a\x30\n\x0eVariablesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\"j\n\nAdjectives\x12-\n\x06values\x18\x01 \x03(\x0b\x32\x1d.agent.Adjectives.ValuesEntry\x1a-\n\x0bValuesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\"\xe2\x02\n\x0bPersonality\x12\x36\n\nadjectives\x18\x01 \x03(\x0b\x32\".agent.Personality.AdjectivesEntry\x12\x0e\n\x06\x63ustom\x18\x02 \x01(\t\x12\x12\n\ncreated_at\x18\x03 \x01(\t\x12\x12\n\ncreated_by\x18\x04 \x01(\t\x12\x33\n\nupdated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x88\x01\x01\x12\x17\n\nupdated_by\x18\x06 \x01(\tH\x01\x88\x01\x01\x12\x35\n\nreferences\x18\x07 \x01(\x0b\x32\x1c.agent.PersonalityReferencesH\x02\x88\x01\x01\x1a\x31\n\x0f\x41\x64jectivesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x42\r\n\x0b_updated_atB\r\n\x0b_updated_byB\r\n\x0b_references\"\xc0\x01\n\x1dPersonality_UpdatePersonality\x12*\n\nadjectives\x18\x01 \x01(\x0b\x32\x11.agent.AdjectivesH\x00\x88\x01\x01\x12\x13\n\x06\x63ustom\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x35\n\nreferences\x18\x03 \x01(\x0b\x32\x1c.agent.PersonalityReferencesH\x02\x88\x01\x01\x42\r\n\x0b_adjectivesB\t\n\x07_customB\r\n\x0b_references\"\x89\x01\n\x15PersonalityReferences\x12>\n\tvariables\x18\x01 \x03(\x0b\x32+.agent.PersonalityReferences.VariablesEntry\x1a\x30\n\x0eVariablesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\"\x91\x02\n\x04Role\x12\r\n\x05value\x18\x01 \x01(\t\x12\x17\n\x0f\x61\x64\x64itional_info\x18\x02 \x01(\t\x12\x0e\n\x06\x63ustom\x18\x03 \x01(\t\x12\x12\n\ncreated_at\x18\x04 \x01(\t\x12\x12\n\ncreated_by\x18\x05 \x01(\t\x12\x33\n\nupdated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x88\x01\x01\x12\x17\n\nupdated_by\x18\x07 \x01(\tH\x01\x88\x01\x01\x12.\n\nreferences\x18\x08 \x01(\x0b\x32\x15.agent.RoleReferencesH\x02\x88\x01\x01\x42\r\n\x0b_updated_atB\r\n\x0b_updated_byB\r\n\x0b_references\"\xc0\x01\n\x0fRole_UpdateRole\x12\x12\n\x05value\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1c\n\x0f\x61\x64\x64itional_info\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x13\n\x06\x63ustom\x18\x03 \x01(\tH\x02\x88\x01\x01\x12.\n\nreferences\x18\x04 \x01(\x0b\x32\x15.agent.RoleReferencesH\x03\x88\x01\x01\x42\x08\n\x06_valueB\x12\n\x10_additional_infoB\t\n\x07_customB\r\n\x0b_references\"{\n\x0eRoleReferences\x12\x37\n\tvariables\x18\x01 \x03(\x0b\x32$.agent.RoleReferences.VariablesEntry\x1a\x30\n\x0eVariablesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\"\x8d\x02\n\x11\x44isclaimerMessage\x12\x12\n\nis_enabled\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\x12(\n\x0cringing_tone\x18\x03 \x01(\x0b\x32\x12.agent.AudioToPlay\x12\x12\n\ncreated_at\x18\x04 \x01(\t\x12\x12\n\ncreated_by\x18\x05 \x01(\t\x12\x33\n\nupdated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x88\x01\x01\x12\x17\n\nupdated_by\x18\x07 \x01(\tH\x01\x88\x01\x01\x12\x15\n\rlanguage_code\x18\x08 \x01(\tB\r\n\x0b_updated_atB\r\n\x0b_updated_by\"\xd2\x01\n)DisclaimerMessage_UpdateDisclaimerMessage\x12\x17\n\nis_enabled\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\x14\n\x07message\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x33\n\x0cringing_tone\x18\x03 \x01(\x0b\x32\x18.agent.UpdateRingingToneH\x02\x88\x01\x01\x12\x15\n\rlanguage_code\x18\x04 \x01(\tB\r\n\x0b_is_enabledB\n\n\x08_messageB\x0f\n\r_ringing_tone\"\x93\x01\n\x11UpdateRingingTone\x12\x14\n\x07s3_path\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1e\n\x11original_filename\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x17\n\npublic_url\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\n\n\x08_s3_pathB\x14\n\x12_original_filenameB\r\n\x0b_public_url\"M\n\x0b\x41udioToPlay\x12\x0f\n\x07s3_path\x18\x01 \x01(\t\x12\x19\n\x11original_filename\x18\x02 \x01(\t\x12\x12\n\npublic_url\x18\x03 \x01(\t\"\xaa\x04\n\x0fRulesReferences\x12,\n\x03sms\x18\x01 \x03(\x0b\x32\x1f.agent.RulesReferences.SmsEntry\x12\x34\n\x07handoff\x18\x02 \x03(\x0b\x32#.agent.RulesReferences.HandoffEntry\x12:\n\nattributes\x18\x03 \x03(\x0b\x32&.agent.RulesReferences.AttributesEntry\x12\x44\n\x0fglobalFunctions\x18\x04 \x03(\x0b\x32+.agent.RulesReferences.GlobalFunctionsEntry\x12\x38\n\tvariables\x18\x05 \x03(\x0b\x32%.agent.RulesReferences.VariablesEntry\x1a*\n\x08SmsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a.\n\x0cHandoffEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x31\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x36\n\x14GlobalFunctionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x30\n\x0eVariablesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\"\xda\x01\n\x05Rules\x12\x11\n\tbehaviour\x18\x01 \x01(\t\x12*\n\nreferences\x18\x02 \x01(\x0b\x32\x16.agent.RulesReferences\x12\x12\n\ncreated_at\x18\x03 \x01(\t\x12\x12\n\ncreated_by\x18\x04 \x01(\t\x12\x33\n\nupdated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x88\x01\x01\x12\x17\n\nupdated_by\x18\x06 \x01(\tH\x01\x88\x01\x01\x42\r\n\x0b_updated_atB\r\n\x0b_updated_by\"y\n\x11Rules_UpdateRules\x12\x16\n\tbehaviour\x18\x01 \x01(\tH\x00\x88\x01\x01\x12/\n\nreferences\x18\x02 \x01(\x0b\x32\x16.agent.RulesReferencesH\x01\x88\x01\x01\x42\x0c\n\n_behaviourB\r\n\x0b_references\"\xec\x04\n\x1aLanguageBehaviorReferences\x12\x37\n\x03sms\x18\x01 \x03(\x0b\x32*.agent.LanguageBehaviorReferences.SmsEntry\x12?\n\x07handoff\x18\x02 \x03(\x0b\x32..agent.LanguageBehaviorReferences.HandoffEntry\x12\x45\n\nattributes\x18\x03 \x03(\x0b\x32\x31.agent.LanguageBehaviorReferences.AttributesEntry\x12O\n\x0fglobalFunctions\x18\x04 \x03(\x0b\x32\x36.agent.LanguageBehaviorReferences.GlobalFunctionsEntry\x12\x43\n\tvariables\x18\x05 \x03(\x0b\x32\x30.agent.LanguageBehaviorReferences.VariablesEntry\x1a*\n\x08SmsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a.\n\x0cHandoffEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x31\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x36\n\x14GlobalFunctionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x30\n\x0eVariablesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\"\xa9\x02\n\x10LanguageBehavior\x12\x17\n\x0f\x62\x65havior_prompt\x18\x01 \x01(\t\x12\x15\n\rlanguage_code\x18\x02 \x01(\t\x12.\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\ncreated_by\x18\x04 \x01(\t\x12\x33\n\nupdated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x88\x01\x01\x12\x17\n\nupdated_by\x18\x06 \x01(\tH\x01\x88\x01\x01\x12\x35\n\nreferences\x18\x07 \x01(\x0b\x32!.agent.LanguageBehaviorReferencesB\r\n\x0b_updated_atB\r\n\x0b_updated_by\"\xbd\x01\n\'LanguageBehavior_UpdateLanguageBehavior\x12\x15\n\rlanguage_code\x18\x01 \x01(\t\x12\x1c\n\x0f\x62\x65havior_prompt\x18\x02 \x01(\tH\x00\x88\x01\x01\x12:\n\nreferences\x18\x03 \x01(\x0b\x32!.agent.LanguageBehaviorReferencesH\x01\x88\x01\x01\x42\x12\n\x10_behavior_promptB\r\n\x0b_referencesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'agent_settings_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_AGENTSETTINGS_LANGUAGEBEHAVIORSENTRY']._options = None
  _globals['_AGENTSETTINGS_LANGUAGEBEHAVIORSENTRY']._serialized_options = b'8\001'
  _globals['_GREETINGREFERENCES_VARIABLESENTRY']._options = None
  _globals['_GREETINGREFERENCES_VARIABLESENTRY']._serialized_options = b'8\001'
  _globals['_ADJECTIVES_VALUESENTRY']._options = None
  _globals['_ADJECTIVES_VALUESENTRY']._serialized_options = b'8\001'
  _globals['_PERSONALITY_ADJECTIVESENTRY']._options = None
  _globals['_PERSONALITY_ADJECTIVESENTRY']._serialized_options = b'8\001'
  _globals['_PERSONALITYREFERENCES_VARIABLESENTRY']._options = None
  _globals['_PERSONALITYREFERENCES_VARIABLESENTRY']._serialized_options = b'8\001'
  _globals['_ROLEREFERENCES_VARIABLESENTRY']._options = None
  _globals['_ROLEREFERENCES_VARIABLESENTRY']._serialized_options = b'8\001'
  _globals['_RULESREFERENCES_SMSENTRY']._options = None
  _globals['_RULESREFERENCES_SMSENTRY']._serialized_options = b'8\001'
  _globals['_RULESREFERENCES_HANDOFFENTRY']._options = None
  _globals['_RULESREFERENCES_HANDOFFENTRY']._serialized_options = b'8\001'
  _globals['_RULESREFERENCES_ATTRIBUTESENTRY']._options = None
  _globals['_RULESREFERENCES_ATTRIBUTESENTRY']._serialized_options = b'8\001'
  _globals['_RULESREFERENCES_GLOBALFUNCTIONSENTRY']._options = None
  _globals['_RULESREFERENCES_GLOBALFUNCTIONSENTRY']._serialized_options = b'8\001'
  _globals['_RULESREFERENCES_VARIABLESENTRY']._options = None
  _globals['_RULESREFERENCES_VARIABLESENTRY']._serialized_options = b'8\001'
  _globals['_LANGUAGEBEHAVIORREFERENCES_SMSENTRY']._options = None
  _globals['_LANGUAGEBEHAVIORREFERENCES_SMSENTRY']._serialized_options = b'8\001'
  _globals['_LANGUAGEBEHAVIORREFERENCES_HANDOFFENTRY']._options = None
  _globals['_LANGUAGEBEHAVIORREFERENCES_HANDOFFENTRY']._serialized_options = b'8\001'
  _globals['_LANGUAGEBEHAVIORREFERENCES_ATTRIBUTESENTRY']._options = None
  _globals['_LANGUAGEBEHAVIORREFERENCES_ATTRIBUTESENTRY']._serialized_options = b'8\001'
  _globals['_LANGUAGEBEHAVIORREFERENCES_GLOBALFUNCTIONSENTRY']._options = None
  _globals['_LANGUAGEBEHAVIORREFERENCES_GLOBALFUNCTIONSENTRY']._serialized_options = b'8\001'
  _globals['_LANGUAGEBEHAVIORREFERENCES_VARIABLESENTRY']._options = None
  _globals['_LANGUAGEBEHAVIORREFERENCES_VARIABLESENTRY']._serialized_options = b'8\001'
  _globals['_AGENTSETTINGS']._serialized_start=65
  _globals['_AGENTSETTINGS']._serialized_end=422
  _globals['_AGENTSETTINGS_LANGUAGEBEHAVIORSENTRY']._serialized_start=341
  _globals['_AGENTSETTINGS_LANGUAGEBEHAVIORSENTRY']._serialized_end=422
  _globals['_GREETING']._serialized_start=425
  _globals['_GREETING']._serialized_end=698
  _globals['_GREETING_UPDATEGREETING']._serialized_start=701
  _globals['_GREETING_UPDATEGREETING']._serialized_end=866
  _globals['_GREETINGREFERENCES']._serialized_start=869
  _globals['_GREETINGREFERENCES']._serialized_end=1000
  _globals['_GREETINGREFERENCES_VARIABLESENTRY']._serialized_start=952
  _globals['_GREETINGREFERENCES_VARIABLESENTRY']._serialized_end=1000
  _globals['_ADJECTIVES']._serialized_start=1002
  _globals['_ADJECTIVES']._serialized_end=1108
  _globals['_ADJECTIVES_VALUESENTRY']._serialized_start=1063
  _globals['_ADJECTIVES_VALUESENTRY']._serialized_end=1108
  _globals['_PERSONALITY']._serialized_start=1111
  _globals['_PERSONALITY']._serialized_end=1465
  _globals['_PERSONALITY_ADJECTIVESENTRY']._serialized_start=1371
  _globals['_PERSONALITY_ADJECTIVESENTRY']._serialized_end=1420
  _globals['_PERSONALITY_UPDATEPERSONALITY']._serialized_start=1468
  _globals['_PERSONALITY_UPDATEPERSONALITY']._serialized_end=1660
  _globals['_PERSONALITYREFERENCES']._serialized_start=1663
  _globals['_PERSONALITYREFERENCES']._serialized_end=1800
  _globals['_PERSONALITYREFERENCES_VARIABLESENTRY']._serialized_start=952
  _globals['_PERSONALITYREFERENCES_VARIABLESENTRY']._serialized_end=1000
  _globals['_ROLE']._serialized_start=1803
  _globals['_ROLE']._serialized_end=2076
  _globals['_ROLE_UPDATEROLE']._serialized_start=2079
  _globals['_ROLE_UPDATEROLE']._serialized_end=2271
  _globals['_ROLEREFERENCES']._serialized_start=2273
  _globals['_ROLEREFERENCES']._serialized_end=2396
  _globals['_ROLEREFERENCES_VARIABLESENTRY']._serialized_start=952
  _globals['_ROLEREFERENCES_VARIABLESENTRY']._serialized_end=1000
  _globals['_DISCLAIMERMESSAGE']._serialized_start=2399
  _globals['_DISCLAIMERMESSAGE']._serialized_end=2668
  _globals['_DISCLAIMERMESSAGE_UPDATEDISCLAIMERMESSAGE']._serialized_start=2671
  _globals['_DISCLAIMERMESSAGE_UPDATEDISCLAIMERMESSAGE']._serialized_end=2881
  _globals['_UPDATERINGINGTONE']._serialized_start=2884
  _globals['_UPDATERINGINGTONE']._serialized_end=3031
  _globals['_AUDIOTOPLAY']._serialized_start=3033
  _globals['_AUDIOTOPLAY']._serialized_end=3110
  _globals['_RULESREFERENCES']._serialized_start=3113
  _globals['_RULESREFERENCES']._serialized_end=3667
  _globals['_RULESREFERENCES_SMSENTRY']._serialized_start=3420
  _globals['_RULESREFERENCES_SMSENTRY']._serialized_end=3462
  _globals['_RULESREFERENCES_HANDOFFENTRY']._serialized_start=3464
  _globals['_RULESREFERENCES_HANDOFFENTRY']._serialized_end=3510
  _globals['_RULESREFERENCES_ATTRIBUTESENTRY']._serialized_start=3512
  _globals['_RULESREFERENCES_ATTRIBUTESENTRY']._serialized_end=3561
  _globals['_RULESREFERENCES_GLOBALFUNCTIONSENTRY']._serialized_start=3563
  _globals['_RULESREFERENCES_GLOBALFUNCTIONSENTRY']._serialized_end=3617
  _globals['_RULESREFERENCES_VARIABLESENTRY']._serialized_start=952
  _globals['_RULESREFERENCES_VARIABLESENTRY']._serialized_end=1000
  _globals['_RULES']._serialized_start=3670
  _globals['_RULES']._serialized_end=3888
  _globals['_RULES_UPDATERULES']._serialized_start=3890
  _globals['_RULES_UPDATERULES']._serialized_end=4011
  _globals['_LANGUAGEBEHAVIORREFERENCES']._serialized_start=4014
  _globals['_LANGUAGEBEHAVIORREFERENCES']._serialized_end=4634
  _globals['_LANGUAGEBEHAVIORREFERENCES_SMSENTRY']._serialized_start=3420
  _globals['_LANGUAGEBEHAVIORREFERENCES_SMSENTRY']._serialized_end=3462
  _globals['_LANGUAGEBEHAVIORREFERENCES_HANDOFFENTRY']._serialized_start=3464
  _globals['_LANGUAGEBEHAVIORREFERENCES_HANDOFFENTRY']._serialized_end=3510
  _globals['_LANGUAGEBEHAVIORREFERENCES_ATTRIBUTESENTRY']._serialized_start=3512
  _globals['_LANGUAGEBEHAVIORREFERENCES_ATTRIBUTESENTRY']._serialized_end=3561
  _globals['_LANGUAGEBEHAVIORREFERENCES_GLOBALFUNCTIONSENTRY']._serialized_start=3563
  _globals['_LANGUAGEBEHAVIORREFERENCES_GLOBALFUNCTIONSENTRY']._serialized_end=3617
  _globals['_LANGUAGEBEHAVIORREFERENCES_VARIABLESENTRY']._serialized_start=952
  _globals['_LANGUAGEBEHAVIORREFERENCES_VARIABLESENTRY']._serialized_end=1000
  _globals['_LANGUAGEBEHAVIOR']._serialized_start=4637
  _globals['_LANGUAGEBEHAVIOR']._serialized_end=4934
  _globals['_LANGUAGEBEHAVIOR_UPDATELANGUAGEBEHAVIOR']._serialized_start=4937
  _globals['_LANGUAGEBEHAVIOR_UPDATELANGUAGEBEHAVIOR']._serialized_end=5126
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: poly_platform/ragdoll/pb/ragdoll_service_grpc.proto
# Protobuf Python Version: 4.25.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from poly_platform.ragdoll.pb import ragdoll_types_pb2 as poly__platform_dot_ragdoll_dot_pb_dot_ragdoll__types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3poly_platform/ragdoll/pb/ragdoll_service_grpc.proto\x12\x07ragdoll\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,poly_platform/ragdoll/pb/ragdoll_types.proto\":\n\x05\x45rror\x12 \n\x04\x63ode\x18\x01 \x01(\x0e\x32\x12.ragdoll.ErrorCode\x12\x0f\n\x07message\x18\x02 \x01(\t\"K\n\x10ListTopicsParams\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\"\x91\x01\n\x10ListTopicsOutput\x12\x1e\n\x06topics\x18\x01 \x03(\x0b\x32\x0e.ragdoll.Topic\x12-\n\x15uninstantiated_topics\x18\x03 \x03(\x0b\x32\x0e.ragdoll.Topic\x12.\n\nupdated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"u\n\x12ListTopicsResponse\x12+\n\x06output\x18\x01 \x01(\x0b\x32\x19.ragdoll.ListTopicsOutputH\x00\x12\x1f\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x0e.ragdoll.ErrorH\x00\x42\x11\n\x0foutput_or_error\"\xaa\x01\n\x11QueryTopicsParams\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\x12\x12\n\nquery_text\x18\x04 \x01(\t\x12\r\n\x05top_k\x18\x05 \x01(\r\x12\x11\n\tthreshold\x18\x06 \x01(\x02\x12\x12\n\nvariant_id\x18\x07 \x01(\t\x12\x12\n\nclient_env\x18\x08 \x01(\t\"C\n\x10QueryTopicResult\x12\x1d\n\x05topic\x18\x01 \x01(\x0b\x32\x0e.ragdoll.Topic\x12\x10\n\x08\x64istance\x18\x02 \x01(\x02\"Y\n\x11QueryTopicsOutput\x12*\n\x07results\x18\x01 \x03(\x0b\x32\x19.ragdoll.QueryTopicResult\x12\x18\n\x10required_loading\x18\x02 \x01(\x08\"w\n\x13QueryTopicsResponse\x12,\n\x06output\x18\x01 \x01(\x0b\x32\x1a.ragdoll.QueryTopicsOutputH\x00\x12\x1f\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x0e.ragdoll.ErrorH\x00\x42\x11\n\x0foutput_or_error\"`\n\x11\x45nsureReadyParams\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\x12\x12\n\nclient_env\x18\x04 \x01(\t\"-\n\x11\x45nsureReadyOutput\x12\x18\n\x10required_loading\x18\x01 \x01(\x08\"w\n\x13\x45nsureReadyResponse\x12,\n\x06output\x18\x01 \x01(\x0b\x32\x1a.ragdoll.EnsureReadyOutputH\x00\x12\x1f\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x0e.ragdoll.ErrorH\x00\x42\x11\n\x0foutput_or_error\"\xc4\x02\n\x12UpdateTopicsParams\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\x12\x19\n\x11version_to_create\x18\x04 \x01(\t\x12\x0f\n\x07\x64ry_run\x18\x05 \x01(\x08\x12%\n\rcreate_topics\x18\x06 \x03(\x0b\x32\x0e.ragdoll.Topic\x12%\n\rupdate_topics\x18\x07 \x03(\x0b\x32\x0e.ragdoll.Topic\x12\x18\n\x10\x64\x65lete_topic_ids\x18\x08 \x03(\t\x12\x32\n\x0enew_updated_at\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\x15uninstantiated_topics\x18\n \x03(\x0b\x32\x0e.ragdoll.Topic\"\x87\x01\n\x12UpdateTopicsOutput\x12\x13\n\x0bnum_created\x18\x01 \x01(\r\x12\x13\n\x0bnum_deleted\x18\x02 \x01(\r\x12\x13\n\x0bnum_updated\x18\x03 \x01(\r\x12\x32\n\x0enew_updated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"y\n\x14UpdateTopicsResponse\x12-\n\x06output\x18\x01 \x01(\x0b\x32\x1b.ragdoll.UpdateTopicsOutputH\x00\x12\x1f\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x0e.ragdoll.ErrorH\x00\x42\x11\n\x0foutput_or_error\"\xab\x01\n\x15\x43reateEphemeralParams\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\x12\x19\n\x11version_to_create\x18\x04 \x01(\t\x12.\n\nupdated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06\x65xpiry\x18\x06 \x01(\r\"[\n\x15\x43reateEphemeralOutput\x12\x12\n\nindex_size\x18\x01 \x01(\r\x12.\n\nupdated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x7f\n\x17\x43reateEphemeralResponse\x12\x30\n\x06output\x18\x01 \x01(\x0b\x32\x1e.ragdoll.CreateEphemeralOutputH\x00\x12\x1f\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x0e.ragdoll.ErrorH\x00\x42\x11\n\x0foutput_or_error\"\xd2\x02\n\x13UpdateInplaceParams\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\x12\x37\n\x13\x65xpected_updated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x32\n\x0enew_updated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12%\n\rcreate_topics\x18\x06 \x03(\x0b\x32\x0e.ragdoll.Topic\x12%\n\rupdate_topics\x18\x07 \x03(\x0b\x32\x0e.ragdoll.Topic\x12\x18\n\x10\x64\x65lete_topic_ids\x18\x08 \x03(\t\x12-\n\x15uninstantiated_topics\x18\t \x03(\x0b\x32\x0e.ragdoll.Topic\"\xf1\x01\n\x13UpdateInplaceOutput\x12\x13\n\x0bnum_created\x18\x01 \x01(\r\x12\x13\n\x0bnum_deleted\x18\x02 \x01(\r\x12\x13\n\x0bnum_updated\x18\x03 \x01(\r\x12\x32\n\x0enew_updated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12!\n\x19\x63reate_topics_warning_ids\x18\x05 \x03(\t\x12!\n\x19update_topics_warning_ids\x18\x06 \x03(\t\x12!\n\x19\x64\x65lete_topics_warning_ids\x18\x07 \x03(\t\"{\n\x15UpdateInplaceResponse\x12.\n\x06output\x18\x01 \x01(\x0b\x32\x1c.ragdoll.UpdateInplaceOutputH\x00\x12\x1f\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x0e.ragdoll.ErrorH\x00\x42\x11\n\x0foutput_or_error\"L\n\x11GetMetadataParams\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\"W\n\x11GetMetadataOutput\x12\x12\n\nindex_size\x18\x01 \x01(\r\x12.\n\nupdated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"v\n\x12GetMetadataReponse\x12,\n\x06output\x18\x01 \x01(\x0b\x32\x1a.ragdoll.GetMetadataOutputH\x00\x12\x1f\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x0e.ragdoll.ErrorH\x00\x42\x11\n\x0foutput_or_error\"M\n\x12\x44\x65leteCachedParams\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\"\x14\n\x12\x44\x65leteCachedOutput\"y\n\x14\x44\x65leteCachedResponse\x12-\n\x06output\x18\x01 \x01(\x0b\x32\x1b.ragdoll.DeleteCachedOutputH\x00\x12\x1f\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x0e.ragdoll.ErrorH\x00\x42\x11\n\x0foutput_or_error\"e\n\x11UpdateModelParams\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x17\n\x0f\x65mbedding_model\x18\x03 \x01(\t\x12\x0f\n\x07version\x18\x04 \x01(\t\"\x13\n\x11UpdateModelOutput\"w\n\x13UpdateModelResponse\x12,\n\x06output\x18\x01 \x01(\x0b\x32\x1a.ragdoll.UpdateModelOutputH\x00\x12\x1f\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x0e.ragdoll.ErrorH\x00\x42\x11\n\x0foutput_or_error\"F\n\x0bTextToEmbed\x12\n\n\x02id\x18\x01 \x01(\t\x12\x11\n\x04name\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\tB\x07\n\x05_name\"x\n\x10\x45mbedTextsParams\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x17\n\x0f\x65mbedding_model\x18\x03 \x01(\t\x12#\n\x05texts\x18\x04 \x03(\x0b\x32\x14.ragdoll.TextToEmbed\"!\n\x0c\x45mbeddedText\x12\x11\n\tembedding\x18\x01 \x01(\x0c\"A\n\x10\x45mbedTextsOutput\x12-\n\x0e\x65mbedded_texts\x18\x01 \x03(\x0b\x32\x15.ragdoll.EmbeddedText\"u\n\x12\x45mbedTextsResponse\x12+\n\x06output\x18\x01 \x01(\x0b\x32\x19.ragdoll.EmbedTextsOutputH\x00\x12\x1f\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x0e.ragdoll.ErrorH\x00\x42\x11\n\x0foutput_or_error*\x95\x01\n\tErrorCode\x12\x08\n\x04NONE\x10\x00\x12\x1d\n\x19KNOWLEDGE_BASE_NOT_EXISTS\x10\x01\x12\x14\n\x10INVALID_ARGUMENT\x10\x02\x12\x11\n\rREDIS_TIMEOUT\x10\x03\x12\x13\n\x0f\x43OMPUTE_TIMEOUT\x10\x04\x12!\n\x1dKNOWLEDGE_BASE_ALREADY_EXISTS\x10\x05\x32\x99\x06\n\x0eRagdollService\x12G\n\x0blist_topics\x12\x19.ragdoll.ListTopicsParams\x1a\x1b.ragdoll.ListTopicsResponse\"\x00\x12M\n\rupdate_topics\x12\x1b.ragdoll.UpdateTopicsParams\x1a\x1d.ragdoll.UpdateTopicsResponse\"\x00\x12J\n\x0cquery_topics\x12\x1a.ragdoll.QueryTopicsParams\x1a\x1c.ragdoll.QueryTopicsResponse\"\x00\x12J\n\x0c\x65nsure_ready\x12\x1a.ragdoll.EnsureReadyParams\x1a\x1c.ragdoll.EnsureReadyResponse\"\x00\x12V\n\x10\x63reate_ephemeral\x12\x1e.ragdoll.CreateEphemeralParams\x1a .ragdoll.CreateEphemeralResponse\"\x00\x12P\n\x0eupdate_inplace\x12\x1c.ragdoll.UpdateInplaceParams\x1a\x1e.ragdoll.UpdateInplaceResponse\"\x00\x12I\n\x0cget_metadata\x12\x1a.ragdoll.GetMetadataParams\x1a\x1b.ragdoll.GetMetadataReponse\"\x00\x12M\n\rdelete_cached\x12\x1b.ragdoll.DeleteCachedParams\x1a\x1d.ragdoll.DeleteCachedResponse\"\x00\x12J\n\x0cupdate_model\x12\x1a.ragdoll.UpdateModelParams\x1a\x1c.ragdoll.UpdateModelResponse\"\x00\x12G\n\x0b\x65mbed_texts\x12\x19.ragdoll.EmbedTextsParams\x1a\x1b.ragdoll.EmbedTextsResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'poly_platform.ragdoll.pb.ragdoll_service_grpc_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_ERRORCODE']._serialized_start=4133
  _globals['_ERRORCODE']._serialized_end=4282
  _globals['_ERROR']._serialized_start=143
  _globals['_ERROR']._serialized_end=201
  _globals['_LISTTOPICSPARAMS']._serialized_start=203
  _globals['_LISTTOPICSPARAMS']._serialized_end=278
  _globals['_LISTTOPICSOUTPUT']._serialized_start=281
  _globals['_LISTTOPICSOUTPUT']._serialized_end=426
  _globals['_LISTTOPICSRESPONSE']._serialized_start=428
  _globals['_LISTTOPICSRESPONSE']._serialized_end=545
  _globals['_QUERYTOPICSPARAMS']._serialized_start=548
  _globals['_QUERYTOPICSPARAMS']._serialized_end=718
  _globals['_QUERYTOPICRESULT']._serialized_start=720
  _globals['_QUERYTOPICRESULT']._serialized_end=787
  _globals['_QUERYTOPICSOUTPUT']._serialized_start=789
  _globals['_QUERYTOPICSOUTPUT']._serialized_end=878
  _globals['_QUERYTOPICSRESPONSE']._serialized_start=880
  _globals['_QUERYTOPICSRESPONSE']._serialized_end=999
  _globals['_ENSUREREADYPARAMS']._serialized_start=1001
  _globals['_ENSUREREADYPARAMS']._serialized_end=1097
  _globals['_ENSUREREADYOUTPUT']._serialized_start=1099
  _globals['_ENSUREREADYOUTPUT']._serialized_end=1144
  _globals['_ENSUREREADYRESPONSE']._serialized_start=1146
  _globals['_ENSUREREADYRESPONSE']._serialized_end=1265
  _globals['_UPDATETOPICSPARAMS']._serialized_start=1268
  _globals['_UPDATETOPICSPARAMS']._serialized_end=1592
  _globals['_UPDATETOPICSOUTPUT']._serialized_start=1595
  _globals['_UPDATETOPICSOUTPUT']._serialized_end=1730
  _globals['_UPDATETOPICSRESPONSE']._serialized_start=1732
  _globals['_UPDATETOPICSRESPONSE']._serialized_end=1853
  _globals['_CREATEEPHEMERALPARAMS']._serialized_start=1856
  _globals['_CREATEEPHEMERALPARAMS']._serialized_end=2027
  _globals['_CREATEEPHEMERALOUTPUT']._serialized_start=2029
  _globals['_CREATEEPHEMERALOUTPUT']._serialized_end=2120
  _globals['_CREATEEPHEMERALRESPONSE']._serialized_start=2122
  _globals['_CREATEEPHEMERALRESPONSE']._serialized_end=2249
  _globals['_UPDATEINPLACEPARAMS']._serialized_start=2252
  _globals['_UPDATEINPLACEPARAMS']._serialized_end=2590
  _globals['_UPDATEINPLACEOUTPUT']._serialized_start=2593
  _globals['_UPDATEINPLACEOUTPUT']._serialized_end=2834
  _globals['_UPDATEINPLACERESPONSE']._serialized_start=2836
  _globals['_UPDATEINPLACERESPONSE']._serialized_end=2959
  _globals['_GETMETADATAPARAMS']._serialized_start=2961
  _globals['_GETMETADATAPARAMS']._serialized_end=3037
  _globals['_GETMETADATAOUTPUT']._serialized_start=3039
  _globals['_GETMETADATAOUTPUT']._serialized_end=3126
  _globals['_GETMETADATAREPONSE']._serialized_start=3128
  _globals['_GETMETADATAREPONSE']._serialized_end=3246
  _globals['_DELETECACHEDPARAMS']._serialized_start=3248
  _globals['_DELETECACHEDPARAMS']._serialized_end=3325
  _globals['_DELETECACHEDOUTPUT']._serialized_start=3327
  _globals['_DELETECACHEDOUTPUT']._serialized_end=3347
  _globals['_DELETECACHEDRESPONSE']._serialized_start=3349
  _globals['_DELETECACHEDRESPONSE']._serialized_end=3470
  _globals['_UPDATEMODELPARAMS']._serialized_start=3472
  _globals['_UPDATEMODELPARAMS']._serialized_end=3573
  _globals['_UPDATEMODELOUTPUT']._serialized_start=3575
  _globals['_UPDATEMODELOUTPUT']._serialized_end=3594
  _globals['_UPDATEMODELRESPONSE']._serialized_start=3596
  _globals['_UPDATEMODELRESPONSE']._serialized_end=3715
  _globals['_TEXTTOEMBED']._serialized_start=3717
  _globals['_TEXTTOEMBED']._serialized_end=3787
  _globals['_EMBEDTEXTSPARAMS']._serialized_start=3789
  _globals['_EMBEDTEXTSPARAMS']._serialized_end=3909
  _globals['_EMBEDDEDTEXT']._serialized_start=3911
  _globals['_EMBEDDEDTEXT']._serialized_end=3944
  _globals['_EMBEDTEXTSOUTPUT']._serialized_start=3946
  _globals['_EMBEDTEXTSOUTPUT']._serialized_end=4011
  _globals['_EMBEDTEXTSRESPONSE']._serialized_start=4013
  _globals['_EMBEDTEXTSRESPONSE']._serialized_end=4130
  _globals['_RAGDOLLSERVICE']._serialized_start=4285
  _globals['_RAGDOLLSERVICE']._serialized_end=5078
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: asr.proto
# Protobuf Python Version: 4.25.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tasr.proto\x12\x03\x61sr\x1a\x1cgoogle/protobuf/struct.proto\"g\n\x03\x41sr\x12\x13\n\x0bprovider_id\x18\x01 \x01(\t\x12\x10\n\x08model_id\x18\x02 \x01(\t\x12\x10\n\x08language\x18\x03 \x01(\t\x12\'\n\x06\x63onfig\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Structb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'asr_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_ASR']._serialized_start=48
  _globals['_ASR']._serialized_end=151
# @@protoc_insertion_point(module_scope)



"""Util functions for resources in ADK

Copyright PolyAI Limited
"""

import ast
import functools
import hashlib
import inspect
import json
import logging
import os
import re
import subprocess
import sys
from difflib import unified_diff
from enum import Enum
from io import StringIO
from typing import TYPE_CHECKING, Optional

import ruamel.yaml as yaml

logger = logging.getLogger(__name__)

if TYPE_CHECKING:
    from poly.resources.flows import BaseFlowStep
    from poly.resources.resource import ResourceMapping

# Create YAML instances with block style for multiline strings
_yaml_dumper = yaml.YAML()
_yaml_dumper.default_flow_style = False
_yaml_dumper.preserve_quotes = False
_yaml_dumper.width = 100

_yaml_loader = yaml.YAML(typ="safe")
_yaml_loader.preserve_quotes = False


def resource_to_dict(obj) -> dict:
    """Recursively serialize a resource dataclass to a dict.

    Like dataclasses.asdict() but only includes keys that match
    the class's __init__ parameters, so extra dataclass fields
    (e.g. computed or create-only fields) are excluded.
    """
    from dataclasses import fields

    valid_params = set(inspect.signature(type(obj).__init__).parameters.keys()) - {"self"}
    result = {}
    for f in fields(obj):
        if f.name not in valid_params:
            continue
        result[f.name] = _serialize_value(getattr(obj, f.name))
    return result


def _serialize_value(value):
    """Recursively serialize a value for status dict storage."""
    from dataclasses import is_dataclass
    from enum import Enum

    if is_dataclass(value) and not isinstance(value, type):
        return resource_to_dict(value)
    if isinstance(value, Enum):
        return value.value
    if isinstance(value, list):
        return [_serialize_value(item) for item in value]
    if isinstance(value, tuple):
        return tuple(_serialize_value(item) for item in value)
    if isinstance(value, dict):
        return {k: _serialize_value(v) for k, v in value.items()}
    return value


def _key_needs_quoting(key: str) -> bool:
    """Return True if a YAML key should be quoted to avoid parse errors."""
    # & and * are YAML indicators (anchor, alias) that require quoting
    return "&" in key or "*" in key


def _quote_keys_for_yaml(data):
    """Recursively quote dict keys that contain special YAML characters."""
    if isinstance(data, dict):
        result = {}
        for k, v in data.items():
            if isinstance(k, str) and _key_needs_quoting(k):
                new_key = yaml.scalarstring.DoubleQuotedScalarString(k)
            else:
                new_key = k
            result[new_key] = _quote_keys_for_yaml(v)
        return result
    elif isinstance(data, list):
        return [_quote_keys_for_yaml(item) for item in data]
    return data


def set_block_style_for_multiline_strings(data):
    """Recursively set block style for strings containing newlines in ruamel.yaml format.

    Args:
        data: Dictionary, list, or other data structure to process in-place.
    """
    if isinstance(data, dict):
        for key, value in data.items():
            if isinstance(value, str) and "\n" in value:
                data[key] = yaml.scalarstring.LiteralScalarString(value)
            else:
                set_block_style_for_multiline_strings(value)
    elif isinstance(data, list):
        for item in data:
            set_block_style_for_multiline_strings(item)


def dump_yaml(data, stream=None):
    """Dump data to YAML format with block style for multiline strings.

    Args:
        data: Data structure to dump.
        stream: Optional stream to write to. If None, returns a string.

    Returns:
        str: YAML string if stream is None, otherwise None.
    """
    data = _quote_keys_for_yaml(data)
    set_block_style_for_multiline_strings(data)
    if stream is None:
        stream = StringIO()
        _yaml_dumper.dump(data, stream)
        return stream.getvalue()
    else:
        _yaml_dumper.dump(data, stream)
        return None


def load_yaml(content):
    """Load YAML content.

    Args:
        content: YAML string or file-like object.

    Returns:
        Parsed YAML data.
    """
    return _yaml_loader.load(content)


def get_diff(original: str, updated: str) -> str:
    """Get the diff between original and updated strings."""

    original_lines = original.splitlines()
    updated_lines = updated.splitlines()

    diff = unified_diff(
        original_lines,
        updated_lines,
        lineterm="",
        fromfile="original",
        tofile="updated",
    )
    return "\n".join(diff)


def contains_merge_conflict(string: str) -> bool:
    """Check if the string contains merge conflict markers."""
    has_start = False
    has_middle = False
    has_end = False
    for line in string.splitlines():
        if "<<<<<<<" in line:
            has_start = True
        elif "=======" in line and has_start:
            has_middle = True
        elif ">>>>>>>" in line and has_middle:
            has_end = True
    return has_start and has_middle and has_end


REFERENCES_PREFIX_MAP = {
    "sms": "twilio_sms",
    "handoff": "ho",
    "attributes": "attr",
    "transition_functions": "ft",
    "global_functions": "fn",
    "entities": "entity",
    "variables": "vrbl",
}
REFERENCES_PREFIX_MAP_REGEX = {
    reference: re.compile(rf"{{{{{prefix}:([\w-]+)}}}}")
    for reference, prefix in REFERENCES_PREFIX_MAP.items()
}

PREFIX_TO_REFERENCE = {prefix: reference for reference, prefix in REFERENCES_PREFIX_MAP.items()}

# Regex matching any {{prefix:value}} reference in a single pass
_REFERENCE_PATTERN = re.compile(r"\{\{(\w+):([^}]+)\}\}")

# Regex to find variable names defined in function code via conv.state.<name>
CONV_STATE_DOT_NAME = re.compile(r"(?<![\w.])conv\.state\.([a-zA-Z_][a-zA-Z0-9_]*)\b(?!\s*\()")


def remove_comments_from_code(code: str) -> str:
    """Return code without comments"""
    clean_code = []
    for line in code.splitlines():
        # remove comments from the line
        line = line.split("#")[0].strip()
        if line:
            clean_code.append(line)

    return "\n".join(clean_code)


def extract_variable_names_from_code(code: str) -> set[str]:
    """Extract variable names used in code via conv.state.<name>"""
    if not code:
        return set()

    names = set(CONV_STATE_DOT_NAME.findall(code))

    return names


def get_references_from_prompt(
    prompt: str, valid_references: list[str] = None, raise_on_invalid: bool = False
) -> dict[str, dict[str, bool]]:
    """Extract references from a prompt string."""
    references: dict[str, dict[str, bool]] = {}
    for reference, regex in REFERENCES_PREFIX_MAP_REGEX.items():
        if valid_references is not None and reference not in valid_references:
            if not raise_on_invalid:
                continue

        references[reference] = {}
        for match in regex.findall(prompt):
            if raise_on_invalid and reference not in valid_references:
                raise ValueError(
                    f"Invalid reference type: {reference} is not a valid reference type for this resource. Valid references are: {valid_references}"
                )
            references[reference][match] = True

    return references


def validate_references(
    references: dict[str, dict[str, bool]],
    resource_mappings: list["ResourceMapping"],
    flow_name: str = None,
) -> tuple[bool, list[str]]:
    """Validate that all references in the prompt are valid.

    Args:
        references (dict[str, dict[str, bool]]): The references to validate.
        resource_mappings (list["ResourceMapping"]): The resource mappings to use for validation.
        flow_name (str): The flow name to use for validation.

    Returns:
        bool: True if all references are valid, False otherwise.
    """
    looking_for = {
        reference: set(values.keys()) for reference, values in references.items() if values
    }

    if not looking_for:
        return True, []

    # Build set of available (reference_type, resource_id) for O(1) lookups
    available = set()
    for rm in resource_mappings:
        if rm.flow_name not in (None, flow_name):
            continue
        ref_type = PREFIX_TO_REFERENCE.get(rm.resource_prefix)
        if ref_type:
            available.add((ref_type, rm.resource_id))

    invalid_references = []
    for reference, ids in looking_for.items():
        for rid in ids:
            if (reference, rid) not in available:
                invalid_references.append(f"{reference}: {rid}")

    return len(invalid_references) == 0, invalid_references


def replace_resource_ids_with_names(
    prompt: str,
    resource_mappings: list["ResourceMapping"],
    flow_folder_name: str = None,
) -> str:
    """Replace resource IDs with names in the prompt string."""
    # Build dict for O(1) lookups: (prefix, id) -> name
    id_to_name: dict[tuple[str, str], str] = {}
    for rm in resource_mappings:
        if rm.flow_name and clean_name(rm.flow_name) not in (None, flow_folder_name):
            continue
        if rm.resource_prefix:
            id_to_name[(rm.resource_prefix, rm.resource_id)] = rm.resource_name

    def _replacer(match: re.Match) -> str:
        key = (match.group(1), match.group(2))
        if key in id_to_name:
            return f"{{{{{key[0]}:{id_to_name[key]}}}}}"
        return match.group(0)

    return _REFERENCE_PATTERN.sub(_replacer, prompt)


def replace_resource_names_with_ids(
    prompt: str,
    resource_mappings: list["ResourceMapping"],
    flow_folder_name: str = None,
) -> str:
    """Replace resource names with IDs in the prompt string."""
    # Build dict for O(1) lookups: (prefix, name) -> id
    name_to_id: dict[tuple[str, str], str] = {}
    for rm in resource_mappings:
        if rm.flow_name and clean_name(rm.flow_name) not in (None, flow_folder_name):
            continue
        if rm.resource_prefix:
            name_to_id[(rm.resource_prefix, rm.resource_name)] = rm.resource_id

    def _replacer(match: re.Match) -> str:
        key = (match.group(1), match.group(2))
        if key in name_to_id:
            return f"{{{{{key[0]}:{name_to_id[key]}}}}}"
        return match.group(0)

    return _REFERENCE_PATTERN.sub(_replacer, prompt)


def get_flow_id_from_flow_name(
    formatted_flow_name: str, resource_mappings: list["ResourceMapping"]
) -> tuple[Optional[str], Optional[str]]:
    """Get the flow ID corresponding to the given flow name from resource mappings."""
    for res in resource_mappings:
        if (
            res.resource_type.__name__ == "FlowConfig"
            and clean_name(res.flow_name) == formatted_flow_name
        ):
            return res.resource_id, res.resource_name
    return None, None


def get_flow_name_from_path(file_path: str) -> Optional[str]:
    """Extract the flow name from the file path."""
    parts = file_path.split(os.sep)
    if "flows" in parts:
        flow_index = parts.index("flows")
        if flow_index < len(parts) - 1:
            flow_name = parts[flow_index + 1]
            return flow_name
    return None


def compute_hash(content: str) -> str:
    """Compute a hash of the given content string."""
    return hashlib.sha256(content.encode("utf-8")).hexdigest()


def _canonicalize_for_hash(obj) -> str | int | float | bool | list | dict:
    """Convert to JSON-serializable form for deterministic hashing."""
    if isinstance(obj, dict):
        return {k: _canonicalize_for_hash(v) for k, v in sorted(obj.items())}
    if isinstance(obj, list):
        return [_canonicalize_for_hash(item) for item in obj]
    if isinstance(obj, (str, int, float, bool, type(None))):
        return obj
    if hasattr(obj, "isoformat"):
        return obj.isoformat()
    if isinstance(obj, Enum):
        return obj.value
    return str(obj)


def compute_hash_from_dict(data: dict) -> str:
    """Compute a hash of a dict without YAML serialization. Faster than compute_hash(dump_yaml(data))."""
    canonical = _canonicalize_for_hash(data)
    return hashlib.sha256(
        json.dumps(canonical, sort_keys=True, default=str).encode("utf-8")
    ).hexdigest()


def _format_def_line(line: str) -> str:
    line = line.strip()

    # remove spaces just inside parentheses
    line = re.sub(r"\(\s+", "(", line)
    line = re.sub(r"\s+\)", ")", line)

    # remove space before colon
    line = re.sub(r"\s+:", ":", line)

    # normalize multiple spaces everywhere else
    line = re.sub(r"\s+", " ", line)

    # remove comma after last parameter
    line = re.sub(r",\):$", "):", line)

    return line + "\n"


def restore_function_def_line(file_content: str, file_name: str) -> str:
    """Restore the main function definition to be formatted on one line.

    In AS, the main function is the same as the file name and is required
    to be formatted on one line. This function restores the main function
    definition to be formatted on one line.

    Args:
        file_content (str): The file content
        file_name (str): The name of the file and main function to be formatted

    Returns:
        str: The formatted file content
    """

    try:
        tree = ast.parse(file_content)
    except SyntaxError:
        return file_content

    fn = None
    for node in ast.walk(tree):
        if isinstance(node, ast.FunctionDef) and node.name == file_name:
            fn = node
            break

    if fn is None:
        return file_content

    lines = file_content.splitlines(keepends=True)
    i = fn.lineno - 1
    header_lines = []
    end_line = None
    for j in range(i, len(lines)):
        header_lines.append(lines[j])
        if lines[j].rstrip().endswith(":"):
            end_line = j
            break

    if end_line is None:
        return file_content

    one_line_header = " ".join(header_lines)
    formatted_one_line_header = _format_def_line(one_line_header)

    return "".join(lines[:i]) + formatted_one_line_header + "".join(lines[end_line + 1 :])


def format_code_with_ruff(code: str, file_name: str) -> str:
    """Format the given code string using ruff (via current Python; no PATH required).

    Uses the same interpreter as the running process, so ruff must be installed
    in that environment (e.g. as a dependency of this package).

    Args:
        code (str): The code to format.
        file_name (str): The name of the file being formatted.

    Returns:
        str: The formatted code.
    """
    ruff_cmd = [sys.executable, "-m", "ruff"]

    # Run ruff format
    process = subprocess.run(
        ruff_cmd + ["format", "-"],
        input=code.encode(),
        capture_output=True,
    )
    if process.returncode != 0:
        logger.error(f"Ruff formatting failed for {file_name}. " + process.stderr.decode())
        return code  # Return original code if ruff fails

    output = process.stdout.decode()

    # Run ruff check --fix to apply any additional fixes
    process = subprocess.run(
        ruff_cmd + ["check", "--fix", "-"],
        input=output.encode(),
        capture_output=True,
    )

    if process.returncode != 0:
        logger.error(f"Ruff check --fix failed for {file_name}. " + process.stderr.decode())
        return output  # Return code after format if check fails
    output = process.stdout.decode()

    return output


def format_yaml(yaml_content: str, file_name: str) -> str:
    """Format YAML content using ruamel.yaml (no external tools like prettier).

    Args:
        yaml_content (str): The YAML content to format.
        file_name (str): The name of the file (unused; for API compatibility).

    Returns:
        str: The formatted YAML content.
    """
    return format_yaml_python(yaml_content)


def format_yaml_python(yaml_content: str) -> str:
    """Format YAML content using ruamel.yaml (no external tools).

    Args:
        yaml_content: Raw YAML string.

    Returns:
        Formatted YAML string, or original on parse error.
    """
    try:
        data = load_yaml(yaml_content)
        if data is None:
            return yaml_content
        return dump_yaml(data)
    except Exception:
        return yaml_content


def format_json_python(json_content: str) -> str:
    """Format JSON content (no external tools).

    Args:
        json_content: Raw JSON string.

    Returns:
        Formatted JSON string (indent=2), or original on parse error.
    """
    try:
        data = json.loads(json_content)
        return json.dumps(data, indent=2) + "\n"
    except Exception:
        return json_content


SPACES_REGEX = re.compile(r"\s+")
NON_LETTER_REGEX = re.compile(r"[^\w\s]", re.UNICODE)
MULTI_UNDERSCORE_REGEX = re.compile(r"_+")


@functools.cache
def clean_name(name: str, lowercase: bool = True) -> str:
    """Convert a resource name to a folder-friendly format."""
    # Convert to lowercase if requested
    if lowercase:
        name = name.lower()
    # Replace all punctuation with spaces
    name = NON_LETTER_REGEX.sub(" ", name)
    # Replace spaces with underscores
    name = SPACES_REGEX.sub("_", name)
    # Replace multiple underscores with single underscore
    name = MULTI_UNDERSCORE_REGEX.sub("_", name)
    # Trim leading/trailing underscores
    name = name.strip("_")

    return name


def to_camel_case(s: str) -> str:
    """Convert a string to camelCase from snake_case"""
    word = "".join(word.capitalize() for word in s.split("_"))
    return word[0].lower() + word[1:]


def to_snake_case(s: str) -> str:
    """Convert a string to snake_case."""
    return "".join([("_" + c.lower()) if c.isupper() else c for c in s]).lstrip("_")


def convert_keys_to_snake_case(dict_obj: dict) -> dict:
    """Convert config keys to snake_case for consistency."""
    return {to_snake_case(k): v for k, v in dict_obj.items()}


def assign_flow_positions(
    nodes: list["BaseFlowStep"],
    start_node_id: str,
    x_start: float = 0.0,
    y_start: float = 0.0,
    x_gap: float = 600.0,
    y_gap: float = 500.0,
) -> None:
    """Assign positions to flow nodes in a grid layout.

    Args:
        nodes (list[dict]): List of node dictionaries to assign positions to.
        x_start (float): Starting x position.
        y_start (float): Starting y position.
        x_gap (float): Gap between nodes in the x direction.
        y_gap (float): Gap between nodes in the y direction.
    """
    known_positions = [node.position for node in nodes if node.position]

    x_max = x_start
    if known_positions:
        x_max = max(pos["x"] for pos in known_positions)

    # Assign start node position first
    assigning_order: list[BaseFlowStep] = []
    for node in nodes:
        if node.step_id != start_node_id:
            assigning_order.append(node)
        else:
            assigning_order.insert(0, node)

    for node in assigning_order:
        if node.position:
            continue

        if node.step_id == start_node_id and not known_positions:
            node.position = {"x": x_start, "y": y_start}
        else:
            x_max = assign_flow_node_position(
                node,
                nodes,
                visited_node_ids=set(),
                x_start=x_max,
                y_start=y_start,
                x_gap=x_gap,
                y_gap=y_gap,
            )

    # Assign conditions label positions
    for node in nodes:
        if not hasattr(node, "conditions"):
            continue
        for condition in node.conditions:
            if condition.position:
                continue
            # If child_node position condition between parent and child
            if condition.child_step:
                child_node = next((n for n in nodes if n.step_id == condition.child_step), None)
                if child_node and child_node.position and node.position:
                    condition.position = {
                        "x": (node.position["x"] + child_node.position["x"]) / 2,
                        "y": (node.position["y"] + child_node.position["y"]) / 2,
                    }
            elif not condition.exit_flow_position:
                # Collect sibling positions: child nodes and other exit_flow_positions
                sibling_x_positions = []

                # Get positions of child node siblings and other exit_flow_positions
                for other_condition in node.conditions:
                    if other_condition == condition:
                        continue

                    # Check for child node siblings
                    if other_condition.child_step:
                        sibling_node = next(
                            (n for n in nodes if n.step_id == other_condition.child_step),
                            None,
                        )
                        if sibling_node and sibling_node.position:
                            sibling_x_positions.append(sibling_node.position["x"])

                    # Check for other exit_flow_positions
                    if other_condition.exit_flow_position:
                        sibling_x_positions.append(other_condition.exit_flow_position["x"])

                max_x_sibling = (
                    max(sibling_x_positions + [node.position["x"]]) + x_gap
                    if sibling_x_positions
                    else node.position["x"]
                )
                condition.exit_flow_position = {
                    "x": max_x_sibling,
                    "y": node.position["y"] + y_gap,
                }
                condition.position = {
                    "x": (node.position["x"] + condition.exit_flow_position["x"]) / 2,
                    "y": node.position["y"] + y_gap / 2,
                }


def assign_flow_node_position(
    node: "BaseFlowStep",
    nodes: list["BaseFlowStep"],
    visited_node_ids: set[str] = None,
    x_start: float = 0.0,
    y_start: float = 0.0,
    x_gap: float = 600.0,
    y_gap: float = 150.0,
) -> float:
    """Assign positions to flow nodes in a grid layout.

    Args:
        nodes (list[dict]): List of node dictionaries to assign positions to.
        x_start (float): Starting x position.
        y_start (float): Starting y position.
        x_gap (float): Gap between nodes in the x direction.
        y_gap (float): Gap between nodes in the y direction.

    Returns:
        float: The maximum x position assigned.
    """
    if node.position:
        return x_start

    if not visited_node_ids:
        visited_node_ids = set()

    if node.step_id in visited_node_ids:
        # Prevent infinite recursion on cycles
        node.position = {
            "x": x_start + x_gap,
            "y": y_start,
        }
        return node.position["x"]

    # Work out if node has a parent node
    parent_node = None
    max_x_sibling = None
    for potential_parent in nodes:
        if not hasattr(potential_parent, "conditions"):
            continue
        for condition in potential_parent.conditions:
            if condition.child_step == node.step_id:
                parent_node = potential_parent

                if not parent_node.position:
                    # Parent has no position yet, assign it first
                    x_start = assign_flow_node_position(
                        parent_node,
                        nodes,
                        visited_node_ids | {node.step_id},
                        x_start,
                        y_start,
                        x_gap,
                        y_gap,
                    )

                sibling_ids = [
                    c.child_step for c in parent_node.conditions if c.child_step != node.step_id
                ]
                sibling_x_positions = [
                    n.position["x"] for n in nodes if n.step_id in sibling_ids and n.position
                ]
                # Add x positions for end flow conditions
                for c in parent_node.conditions:
                    if c.exit_flow_position and c.child_step not in sibling_ids:
                        sibling_x_positions.append(c.exit_flow_position["x"])

                max_x_sibling = (
                    max(sibling_x_positions + [parent_node.position["x"]]) + x_gap
                    if sibling_x_positions
                    else parent_node.position["x"]
                )
                break

    if parent_node:
        # Position below parent node
        node.position = {
            "x": max_x_sibling,
            "y": parent_node.position["y"] + y_gap,
        }

        return max(x_start, node.position["x"])

    else:
        # Position to the right of the last node
        node.position = {
            "x": x_start + x_gap,
            "y": y_start,
        }
        return node.position["x"]



# Copyright PolyAI Limited
# flake8: noqa
# ruff: noqa
# type: ignore
__all__ = ["AgentResponse", "UserInput"]


class UserInput:
    """Object representing a user turn"""

    def __init__(self, text: str) -> None: ...
    @property
    def text(self) -> str: ...
    @property
    def role(self) -> str: ...
    def to_dict(self) -> dict[str, str]: ...
    def to_string(self) -> str: ...
    def __repr__(self) -> str: ...
    def __eq__(self, other: object) -> bool: ...


class AgentResponse:
    """Object representing an agent turn"""

    def __init__(self, text: str) -> None: ...
    @property
    def text(self) -> str: ...
    @property
    def role(self) -> str: ...
    def to_dict(self) -> dict[str, str]: ...
    def to_string(self) -> str: ...
    def __repr__(self) -> str: ...
    def __eq__(self, other: object) -> bool: ...



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class StopKeywordReferences(_message.Message):
    __slots__ = ("globalFunctions",)
    class GlobalFunctionsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    GLOBALFUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    globalFunctions: _containers.ScalarMap[str, bool]
    def __init__(self, globalFunctions: _Optional[_Mapping[str, bool]] = ...) -> None: ...

class RegularExpressions(_message.Message):
    __slots__ = ("patterns",)
    PATTERNS_FIELD_NUMBER: _ClassVar[int]
    patterns: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, patterns: _Optional[_Iterable[str]] = ...) -> None: ...

class StopKeywords(_message.Message):
    __slots__ = ("filters",)
    FILTERS_FIELD_NUMBER: _ClassVar[int]
    filters: _containers.RepeatedCompositeFieldContainer[StopKeyword]
    def __init__(self, filters: _Optional[_Iterable[_Union[StopKeyword, _Mapping]]] = ...) -> None: ...

class StopKeyword(_message.Message):
    __slots__ = ("id", "title", "description", "regular_expressions", "say_phrase", "references", "created_by", "created_at", "updated_by", "updated_at", "language_code")
    ID_FIELD_NUMBER: _ClassVar[int]
    TITLE_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    REGULAR_EXPRESSIONS_FIELD_NUMBER: _ClassVar[int]
    SAY_PHRASE_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    UPDATED_BY_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    LANGUAGE_CODE_FIELD_NUMBER: _ClassVar[int]
    id: str
    title: str
    description: str
    regular_expressions: _containers.RepeatedScalarFieldContainer[str]
    say_phrase: bool
    references: StopKeywordReferences
    created_by: str
    created_at: _timestamp_pb2.Timestamp
    updated_by: str
    updated_at: _timestamp_pb2.Timestamp
    language_code: str
    def __init__(self, id: _Optional[str] = ..., title: _Optional[str] = ..., description: _Optional[str] = ..., regular_expressions: _Optional[_Iterable[str]] = ..., say_phrase: bool = ..., references: _Optional[_Union[StopKeywordReferences, _Mapping]] = ..., created_by: _Optional[str] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., updated_by: _Optional[str] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., language_code: _Optional[str] = ...) -> None: ...

class StopKeyword_Create(_message.Message):
    __slots__ = ("id", "title", "description", "regular_expressions", "say_phrase", "references", "language_code")
    ID_FIELD_NUMBER: _ClassVar[int]
    TITLE_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    REGULAR_EXPRESSIONS_FIELD_NUMBER: _ClassVar[int]
    SAY_PHRASE_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    LANGUAGE_CODE_FIELD_NUMBER: _ClassVar[int]
    id: str
    title: str
    description: str
    regular_expressions: _containers.RepeatedScalarFieldContainer[str]
    say_phrase: bool
    references: StopKeywordReferences
    language_code: str
    def __init__(self, id: _Optional[str] = ..., title: _Optional[str] = ..., description: _Optional[str] = ..., regular_expressions: _Optional[_Iterable[str]] = ..., say_phrase: bool = ..., references: _Optional[_Union[StopKeywordReferences, _Mapping]] = ..., language_code: _Optional[str] = ...) -> None: ...

class StopKeyword_Update(_message.Message):
    __slots__ = ("id", "title", "description", "regular_expressions", "say_phrase", "references", "language_code")
    ID_FIELD_NUMBER: _ClassVar[int]
    TITLE_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    REGULAR_EXPRESSIONS_FIELD_NUMBER: _ClassVar[int]
    SAY_PHRASE_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    LANGUAGE_CODE_FIELD_NUMBER: _ClassVar[int]
    id: str
    title: str
    description: str
    regular_expressions: _containers.RepeatedScalarFieldContainer[str]
    say_phrase: bool
    references: StopKeywordReferences
    language_code: str
    def __init__(self, id: _Optional[str] = ..., title: _Optional[str] = ..., description: _Optional[str] = ..., regular_expressions: _Optional[_Iterable[str]] = ..., say_phrase: bool = ..., references: _Optional[_Union[StopKeywordReferences, _Mapping]] = ..., language_code: _Optional[str] = ...) -> None: ...

class StopKeyword_Delete(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: entities.proto
# Protobuf Python Version: 4.25.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x65ntities.proto\x12\x08\x65ntities\x1a\x1fgoogle/protobuf/timestamp.proto\"\xfb\x01\n\x10\x45ntityReferences\x12=\n\nflow_steps\x18\x01 \x03(\x0b\x32).entities.EntityReferences.FlowStepsEntry\x12\x42\n\rno_code_steps\x18\x02 \x03(\x0b\x32+.entities.EntityReferences.NoCodeStepsEntry\x1a\x30\n\x0e\x46lowStepsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x32\n\x10NoCodeStepsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\".\n\x08\x45ntities\x12\"\n\x08\x65ntities\x18\x01 \x03(\x0b\x32\x10.entities.Entity\"\xa2\x05\n\x06\x45ntity\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x12\n\ncreated_by\x18\x05 \x01(\t\x12.\n\ncreated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\nupdated_by\x18\x07 \x01(\t\x12.\n\nupdated_at\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nreferences\x18\t \x01(\x0b\x32\x1a.entities.EntityReferences\x12)\n\x07numeric\x18\n \x01(\x0b\x32\x16.entities.NumberConfigH\x00\x12\x34\n\x0c\x61lphanumeric\x18\x0b \x01(\x0b\x32\x1c.entities.AlphanumericConfigH\x00\x12/\n\x04\x65num\x18\x0c \x01(\x0b\x32\x1f.entities.MultipleOptionsConfigH\x00\x12$\n\x04\x64\x61te\x18\r \x01(\x0b\x32\x14.entities.DateConfigH\x00\x12\x33\n\x0cphone_number\x18\x0e \x01(\x0b\x32\x1b.entities.PhoneNumberConfigH\x00\x12$\n\x04time\x18\x0f \x01(\x0b\x32\x14.entities.TimeConfigH\x00\x12*\n\x07\x61\x64\x64ress\x18\x10 \x01(\x0b\x32\x17.entities.AddressConfigH\x00\x12-\n\tfree_text\x18\x11 \x01(\x0b\x32\x18.entities.FreeTextConfigH\x00\x12+\n\x0bname_config\x18\x12 \x01(\x0b\x32\x14.entities.NameConfigH\x00\x42\x08\n\x06\x63onfig\"\xa1\x04\n\rEntity_Create\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12.\n\nreferences\x18\x05 \x01(\x0b\x32\x1a.entities.EntityReferences\x12)\n\x07numeric\x18\x06 \x01(\x0b\x32\x16.entities.NumberConfigH\x00\x12\x34\n\x0c\x61lphanumeric\x18\x07 \x01(\x0b\x32\x1c.entities.AlphanumericConfigH\x00\x12/\n\x04\x65num\x18\x08 \x01(\x0b\x32\x1f.entities.MultipleOptionsConfigH\x00\x12$\n\x04\x64\x61te\x18\t \x01(\x0b\x32\x14.entities.DateConfigH\x00\x12\x33\n\x0cphone_number\x18\n \x01(\x0b\x32\x1b.entities.PhoneNumberConfigH\x00\x12$\n\x04time\x18\x0b \x01(\x0b\x32\x14.entities.TimeConfigH\x00\x12*\n\x07\x61\x64\x64ress\x18\x0c \x01(\x0b\x32\x17.entities.AddressConfigH\x00\x12-\n\tfree_text\x18\r \x01(\x0b\x32\x18.entities.FreeTextConfigH\x00\x12+\n\x0bname_config\x18\x0e \x01(\x0b\x32\x14.entities.NameConfigH\x00\x42\x08\n\x06\x63onfig\"\xa1\x04\n\rEntity_Update\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12.\n\nreferences\x18\x05 \x01(\x0b\x32\x1a.entities.EntityReferences\x12)\n\x07numeric\x18\x06 \x01(\x0b\x32\x16.entities.NumberConfigH\x00\x12\x34\n\x0c\x61lphanumeric\x18\x07 \x01(\x0b\x32\x1c.entities.AlphanumericConfigH\x00\x12/\n\x04\x65num\x18\x08 \x01(\x0b\x32\x1f.entities.MultipleOptionsConfigH\x00\x12$\n\x04\x64\x61te\x18\t \x01(\x0b\x32\x14.entities.DateConfigH\x00\x12\x33\n\x0cphone_number\x18\n \x01(\x0b\x32\x1b.entities.PhoneNumberConfigH\x00\x12$\n\x04time\x18\x0b \x01(\x0b\x32\x14.entities.TimeConfigH\x00\x12*\n\x07\x61\x64\x64ress\x18\x0c \x01(\x0b\x32\x17.entities.AddressConfigH\x00\x12-\n\tfree_text\x18\r \x01(\x0b\x32\x18.entities.FreeTextConfigH\x00\x12+\n\x0bname_config\x18\x0e \x01(\x0b\x32\x14.entities.NameConfigH\x00\x42\x08\n\x06\x63onfig\"\x1b\n\rEntity_Delete\x12\n\n\x02id\x18\x01 \x01(\t\"j\n\x0cNumberConfig\x12\x13\n\x0bhas_decimal\x18\x01 \x01(\x08\x12\x11\n\thas_range\x18\x02 \x01(\x08\x12\x10\n\x03min\x18\x03 \x01(\x02H\x00\x88\x01\x01\x12\x10\n\x03max\x18\x04 \x01(\x02H\x01\x88\x01\x01\x42\x06\n\x04_minB\x06\n\x04_max\"Z\n\x12\x41lphanumericConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x17\n\x0fvalidation_type\x18\x02 \x01(\t\x12\x1a\n\x12regular_expression\x18\x03 \x01(\t\"(\n\x15MultipleOptionsConfig\x12\x0f\n\x07options\x18\x01 \x03(\t\"#\n\nDateConfig\x12\x15\n\rrelative_date\x18\x01 \x01(\x08\";\n\x11PhoneNumberConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x15\n\rcountry_codes\x18\x02 \x03(\t\"C\n\nTimeConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x12\n\nstart_time\x18\x02 \x01(\t\x12\x10\n\x08\x65nd_time\x18\x03 \x01(\t\"\x0f\n\rAddressConfig\"\x10\n\x0e\x46reeTextConfig\"\x0c\n\nNameConfigb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'entities_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_ENTITYREFERENCES_FLOWSTEPSENTRY']._options = None
  _globals['_ENTITYREFERENCES_FLOWSTEPSENTRY']._serialized_options = b'8\001'
  _globals['_ENTITYREFERENCES_NOCODESTEPSENTRY']._options = None
  _globals['_ENTITYREFERENCES_NOCODESTEPSENTRY']._serialized_options = b'8\001'
  _globals['_ENTITYREFERENCES']._serialized_start=62
  _globals['_ENTITYREFERENCES']._serialized_end=313
  _globals['_ENTITYREFERENCES_FLOWSTEPSENTRY']._serialized_start=213
  _globals['_ENTITYREFERENCES_FLOWSTEPSENTRY']._serialized_end=261
  _globals['_ENTITYREFERENCES_NOCODESTEPSENTRY']._serialized_start=263
  _globals['_ENTITYREFERENCES_NOCODESTEPSENTRY']._serialized_end=313
  _globals['_ENTITIES']._serialized_start=315
  _globals['_ENTITIES']._serialized_end=361
  _globals['_ENTITY']._serialized_start=364
  _globals['_ENTITY']._serialized_end=1038
  _globals['_ENTITY_CREATE']._serialized_start=1041
  _globals['_ENTITY_CREATE']._serialized_end=1586
  _globals['_ENTITY_UPDATE']._serialized_start=1589
  _globals['_ENTITY_UPDATE']._serialized_end=2134
  _globals['_ENTITY_DELETE']._serialized_start=2136
  _globals['_ENTITY_DELETE']._serialized_end=2163
  _globals['_NUMBERCONFIG']._serialized_start=2165
  _globals['_NUMBERCONFIG']._serialized_end=2271
  _globals['_ALPHANUMERICCONFIG']._serialized_start=2273
  _globals['_ALPHANUMERICCONFIG']._serialized_end=2363
  _globals['_MULTIPLEOPTIONSCONFIG']._serialized_start=2365
  _globals['_MULTIPLEOPTIONSCONFIG']._serialized_end=2405
  _globals['_DATECONFIG']._serialized_start=2407
  _globals['_DATECONFIG']._serialized_end=2442
  _globals['_PHONENUMBERCONFIG']._serialized_start=2444
  _globals['_PHONENUMBERCONFIG']._serialized_end=2503
  _globals['_TIMECONFIG']._serialized_start=2505
  _globals['_TIMECONFIG']._serialized_end=2572
  _globals['_ADDRESSCONFIG']._serialized_start=2574
  _globals['_ADDRESSCONFIG']._serialized_end=2589
  _globals['_FREETEXTCONFIG']._serialized_start=2591
  _globals['_FREETEXTCONFIG']._serialized_end=2607
  _globals['_NAMECONFIG']._serialized_start=2609
  _globals['_NAMECONFIG']._serialized_end=2621
# @@protoc_insertion_point(module_scope)



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AgentSettings(_message.Message):
    __slots__ = ("greeting", "personality", "role", "disclaimer_message", "rules", "language_behaviors")
    class LanguageBehaviorsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: LanguageBehavior
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[LanguageBehavior, _Mapping]] = ...) -> None: ...
    GREETING_FIELD_NUMBER: _ClassVar[int]
    PERSONALITY_FIELD_NUMBER: _ClassVar[int]
    ROLE_FIELD_NUMBER: _ClassVar[int]
    DISCLAIMER_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    RULES_FIELD_NUMBER: _ClassVar[int]
    LANGUAGE_BEHAVIORS_FIELD_NUMBER: _ClassVar[int]
    greeting: _containers.RepeatedCompositeFieldContainer[Greeting]
    personality: Personality
    role: Role
    disclaimer_message: _containers.RepeatedCompositeFieldContainer[DisclaimerMessage]
    rules: Rules
    language_behaviors: _containers.MessageMap[str, LanguageBehavior]
    def __init__(self, greeting: _Optional[_Iterable[_Union[Greeting, _Mapping]]] = ..., personality: _Optional[_Union[Personality, _Mapping]] = ..., role: _Optional[_Union[Role, _Mapping]] = ..., disclaimer_message: _Optional[_Iterable[_Union[DisclaimerMessage, _Mapping]]] = ..., rules: _Optional[_Union[Rules, _Mapping]] = ..., language_behaviors: _Optional[_Mapping[str, LanguageBehavior]] = ...) -> None: ...

class Greeting(_message.Message):
    __slots__ = ("welcome_message", "created_at", "created_by", "updated_at", "updated_by", "references", "language_code")
    WELCOME_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    UPDATED_BY_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    LANGUAGE_CODE_FIELD_NUMBER: _ClassVar[int]
    welcome_message: str
    created_at: str
    created_by: str
    updated_at: _timestamp_pb2.Timestamp
    updated_by: str
    references: GreetingReferences
    language_code: str
    def __init__(self, welcome_message: _Optional[str] = ..., created_at: _Optional[str] = ..., created_by: _Optional[str] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., updated_by: _Optional[str] = ..., references: _Optional[_Union[GreetingReferences, _Mapping]] = ..., language_code: _Optional[str] = ...) -> None: ...

class Greeting_UpdateGreeting(_message.Message):
    __slots__ = ("welcome_message", "references", "language_code")
    WELCOME_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    LANGUAGE_CODE_FIELD_NUMBER: _ClassVar[int]
    welcome_message: str
    references: GreetingReferences
    language_code: str
    def __init__(self, welcome_message: _Optional[str] = ..., references: _Optional[_Union[GreetingReferences, _Mapping]] = ..., language_code: _Optional[str] = ...) -> None: ...

class GreetingReferences(_message.Message):
    __slots__ = ("variables",)
    class VariablesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    variables: _containers.ScalarMap[str, bool]
    def __init__(self, variables: _Optional[_Mapping[str, bool]] = ...) -> None: ...

class Adjectives(_message.Message):
    __slots__ = ("values",)
    class ValuesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.ScalarMap[str, bool]
    def __init__(self, values: _Optional[_Mapping[str, bool]] = ...) -> None: ...

class Personality(_message.Message):
    __slots__ = ("adjectives", "custom", "created_at", "created_by", "updated_at", "updated_by", "references")
    class AdjectivesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    ADJECTIVES_FIELD_NUMBER: _ClassVar[int]
    CUSTOM_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    UPDATED_BY_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    adjectives: _containers.ScalarMap[str, bool]
    custom: str
    created_at: str
    created_by: str
    updated_at: _timestamp_pb2.Timestamp
    updated_by: str
    references: PersonalityReferences
    def __init__(self, adjectives: _Optional[_Mapping[str, bool]] = ..., custom: _Optional[str] = ..., created_at: _Optional[str] = ..., created_by: _Optional[str] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., updated_by: _Optional[str] = ..., references: _Optional[_Union[PersonalityReferences, _Mapping]] = ...) -> None: ...

class Personality_UpdatePersonality(_message.Message):
    __slots__ = ("adjectives", "custom", "references")
    ADJECTIVES_FIELD_NUMBER: _ClassVar[int]
    CUSTOM_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    adjectives: Adjectives
    custom: str
    references: PersonalityReferences
    def __init__(self, adjectives: _Optional[_Union[Adjectives, _Mapping]] = ..., custom: _Optional[str] = ..., references: _Optional[_Union[PersonalityReferences, _Mapping]] = ...) -> None: ...

class PersonalityReferences(_message.Message):
    __slots__ = ("variables",)
    class VariablesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    variables: _containers.ScalarMap[str, bool]
    def __init__(self, variables: _Optional[_Mapping[str, bool]] = ...) -> None: ...

class Role(_message.Message):
    __slots__ = ("value", "additional_info", "custom", "created_at", "created_by", "updated_at", "updated_by", "references")
    VALUE_FIELD_NUMBER: _ClassVar[int]
    ADDITIONAL_INFO_FIELD_NUMBER: _ClassVar[int]
    CUSTOM_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    UPDATED_BY_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    value: str
    additional_info: str
    custom: str
    created_at: str
    created_by: str
    updated_at: _timestamp_pb2.Timestamp
    updated_by: str
    references: RoleReferences
    def __init__(self, value: _Optional[str] = ..., additional_info: _Optional[str] = ..., custom: _Optional[str] = ..., created_at: _Optional[str] = ..., created_by: _Optional[str] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., updated_by: _Optional[str] = ..., references: _Optional[_Union[RoleReferences, _Mapping]] = ...) -> None: ...

class Role_UpdateRole(_message.Message):
    __slots__ = ("value", "additional_info", "custom", "references")
    VALUE_FIELD_NUMBER: _ClassVar[int]
    ADDITIONAL_INFO_FIELD_NUMBER: _ClassVar[int]
    CUSTOM_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    value: str
    additional_info: str
    custom: str
    references: RoleReferences
    def __init__(self, value: _Optional[str] = ..., additional_info: _Optional[str] = ..., custom: _Optional[str] = ..., references: _Optional[_Union[RoleReferences, _Mapping]] = ...) -> None: ...

class RoleReferences(_message.Message):
    __slots__ = ("variables",)
    class VariablesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    variables: _containers.ScalarMap[str, bool]
    def __init__(self, variables: _Optional[_Mapping[str, bool]] = ...) -> None: ...

class DisclaimerMessage(_message.Message):
    __slots__ = ("is_enabled", "message", "ringing_tone", "created_at", "created_by", "updated_at", "updated_by", "language_code")
    IS_ENABLED_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    RINGING_TONE_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    UPDATED_BY_FIELD_NUMBER: _ClassVar[int]
    LANGUAGE_CODE_FIELD_NUMBER: _ClassVar[int]
    is_enabled: bool
    message: str
    ringing_tone: AudioToPlay
    created_at: str
    created_by: str
    updated_at: _timestamp_pb2.Timestamp
    updated_by: str
    language_code: str
    def __init__(self, is_enabled: bool = ..., message: _Optional[str] = ..., ringing_tone: _Optional[_Union[AudioToPlay, _Mapping]] = ..., created_at: _Optional[str] = ..., created_by: _Optional[str] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., updated_by: _Optional[str] = ..., language_code: _Optional[str] = ...) -> None: ...

class DisclaimerMessage_UpdateDisclaimerMessage(_message.Message):
    __slots__ = ("is_enabled", "message", "ringing_tone", "language_code")
    IS_ENABLED_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    RINGING_TONE_FIELD_NUMBER: _ClassVar[int]
    LANGUAGE_CODE_FIELD_NUMBER: _ClassVar[int]
    is_enabled: bool
    message: str
    ringing_tone: UpdateRingingTone
    language_code: str
    def __init__(self, is_enabled: bool = ..., message: _Optional[str] = ..., ringing_tone: _Optional[_Union[UpdateRingingTone, _Mapping]] = ..., language_code: _Optional[str] = ...) -> None: ...

class UpdateRingingTone(_message.Message):
    __slots__ = ("s3_path", "original_filename", "public_url")
    S3_PATH_FIELD_NUMBER: _ClassVar[int]
    ORIGINAL_FILENAME_FIELD_NUMBER: _ClassVar[int]
    PUBLIC_URL_FIELD_NUMBER: _ClassVar[int]
    s3_path: str
    original_filename: str
    public_url: str
    def __init__(self, s3_path: _Optional[str] = ..., original_filename: _Optional[str] = ..., public_url: _Optional[str] = ...) -> None: ...

class AudioToPlay(_message.Message):
    __slots__ = ("s3_path", "original_filename", "public_url")
    S3_PATH_FIELD_NUMBER: _ClassVar[int]
    ORIGINAL_FILENAME_FIELD_NUMBER: _ClassVar[int]
    PUBLIC_URL_FIELD_NUMBER: _ClassVar[int]
    s3_path: str
    original_filename: str
    public_url: str
    def __init__(self, s3_path: _Optional[str] = ..., original_filename: _Optional[str] = ..., public_url: _Optional[str] = ...) -> None: ...

class RulesReferences(_message.Message):
    __slots__ = ("sms", "handoff", "attributes", "globalFunctions", "variables")
    class SmsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class HandoffEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class AttributesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class GlobalFunctionsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class VariablesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    SMS_FIELD_NUMBER: _ClassVar[int]
    HANDOFF_FIELD_NUMBER: _ClassVar[int]
    ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    GLOBALFUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    sms: _containers.ScalarMap[str, bool]
    handoff: _containers.ScalarMap[str, bool]
    attributes: _containers.ScalarMap[str, bool]
    globalFunctions: _containers.ScalarMap[str, bool]
    variables: _containers.ScalarMap[str, bool]
    def __init__(self, sms: _Optional[_Mapping[str, bool]] = ..., handoff: _Optional[_Mapping[str, bool]] = ..., attributes: _Optional[_Mapping[str, bool]] = ..., globalFunctions: _Optional[_Mapping[str, bool]] = ..., variables: _Optional[_Mapping[str, bool]] = ...) -> None: ...

class Rules(_message.Message):
    __slots__ = ("behaviour", "references", "created_at", "created_by", "updated_at", "updated_by")
    BEHAVIOUR_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    UPDATED_BY_FIELD_NUMBER: _ClassVar[int]
    behaviour: str
    references: RulesReferences
    created_at: str
    created_by: str
    updated_at: _timestamp_pb2.Timestamp
    updated_by: str
    def __init__(self, behaviour: _Optional[str] = ..., references: _Optional[_Union[RulesReferences, _Mapping]] = ..., created_at: _Optional[str] = ..., created_by: _Optional[str] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., updated_by: _Optional[str] = ...) -> None: ...

class Rules_UpdateRules(_message.Message):
    __slots__ = ("behaviour", "references")
    BEHAVIOUR_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    behaviour: str
    references: RulesReferences
    def __init__(self, behaviour: _Optional[str] = ..., references: _Optional[_Union[RulesReferences, _Mapping]] = ...) -> None: ...

class LanguageBehaviorReferences(_message.Message):
    __slots__ = ("sms", "handoff", "attributes", "globalFunctions", "variables")
    class SmsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class HandoffEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class AttributesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class GlobalFunctionsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class VariablesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    SMS_FIELD_NUMBER: _ClassVar[int]
    HANDOFF_FIELD_NUMBER: _ClassVar[int]
    ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    GLOBALFUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    sms: _containers.ScalarMap[str, bool]
    handoff: _containers.ScalarMap[str, bool]
    attributes: _containers.ScalarMap[str, bool]
    globalFunctions: _containers.ScalarMap[str, bool]
    variables: _containers.ScalarMap[str, bool]
    def __init__(self, sms: _Optional[_Mapping[str, bool]] = ..., handoff: _Optional[_Mapping[str, bool]] = ..., attributes: _Optional[_Mapping[str, bool]] = ..., globalFunctions: _Optional[_Mapping[str, bool]] = ..., variables: _Optional[_Mapping[str, bool]] = ...) -> None: ...

class LanguageBehavior(_message.Message):
    __slots__ = ("behavior_prompt", "language_code", "created_at", "created_by", "updated_at", "updated_by", "references")
    BEHAVIOR_PROMPT_FIELD_NUMBER: _ClassVar[int]
    LANGUAGE_CODE_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    UPDATED_BY_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    behavior_prompt: str
    language_code: str
    created_at: _timestamp_pb2.Timestamp
    created_by: str
    updated_at: _timestamp_pb2.Timestamp
    updated_by: str
    references: LanguageBehaviorReferences
    def __init__(self, behavior_prompt: _Optional[str] = ..., language_code: _Optional[str] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., created_by: _Optional[str] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., updated_by: _Optional[str] = ..., references: _Optional[_Union[LanguageBehaviorReferences, _Mapping]] = ...) -> None: ...

class LanguageBehavior_UpdateLanguageBehavior(_message.Message):
    __slots__ = ("language_code", "behavior_prompt", "references")
    LANGUAGE_CODE_FIELD_NUMBER: _ClassVar[int]
    BEHAVIOR_PROMPT_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    language_code: str
    behavior_prompt: str
    references: LanguageBehaviorReferences
    def __init__(self, language_code: _Optional[str] = ..., behavior_prompt: _Optional[str] = ..., references: _Optional[_Union[LanguageBehaviorReferences, _Mapping]] = ...) -> None: ...



"""Client for the Agent Studio Platform API

Copyright PolyAI Limited
"""

import json
import logging
import os
import typing as ty
import uuid

import requests

logger = logging.getLogger(__name__)

ACCOUNTS_URL = "/accounts"
PROJECTS_URL = "/accounts/{account_id}/projects"
DEPLOYMENTS_URL = "/accounts/{account_id}/projects/{project_id}/deployments"
ACTIVE_DEPLOYMENTS_URL = "/accounts/{account_id}/projects/{project_id}/deployments/active"
CHAT_URL = "/accounts/{account_id}/projects/{project_id}/chat"
CHAT_CONVERSATION_URL = "/accounts/{account_id}/projects/{project_id}/chat/{conversation_id}"
DRAFT_CHAT_URL = "/accounts/{account_id}/projects/{project_id}/draft/chat"
DRAFT_CHAT_CONVERSATION_URL = (
    "/accounts/{account_id}/projects/{project_id}/draft/chat/{conversation_id}"
)
CHAT_END_URL = "/accounts/{account_id}/projects/{project_id}/chat/{conversation_id}/end"


class PlatformAPIHandler:
    """Class for interacting with the Platform API"""

    region_to_base_url = {
        "dev": "https://api.dev.poly.ai/adk/v1",
        "staging": "https://api.staging.poly.ai/adk/v1",
        "euw-1": "https://api.eu.poly.ai/adk/v1",
        "uk-1": "https://api.uk.poly.ai/adk/v1",
        "us-1": "https://api.us.poly.ai/adk/v1",
    }

    @staticmethod
    def get_base_url(region: str) -> str:
        """Get the base URL for the Platform API based on the region.

        Args:
            region (str): The region name
        Returns:
            str: The base URL for the Platform API
        """
        if base_url := PlatformAPIHandler.region_to_base_url.get(region):
            return base_url
        raise ValueError(f"Unknown region: {region}")

    @staticmethod
    def _retrieve_api_key() -> str:
        """Get API key from environment"""
        try:
            return os.getenv("POLY_ADK_KEY")
        except Exception:
            raise ValueError("POLY_ADK_KEY environment variable is required")

    @staticmethod
    def make_request(
        region: str,
        endpoint: str,
        method: str = "GET",
        data: ty.Optional[dict] = None,
        params: ty.Optional[dict] = None,
    ) -> dict:
        """Make a request to the Platform API.

        Args:
            region (str): The region name
            endpoint (str): The API endpoint
            method (str): The HTTP method
            data (dict | None): The request body for POST/PUT requests
            params (dict | None): Query string parameters

        Returns:
            dict: The response JSON
        """
        url = PlatformAPIHandler.get_base_url(region) + endpoint
        correlation_id = f"adk-{uuid.uuid4()}"

        headers = {
            "X-API-KEY": PlatformAPIHandler._retrieve_api_key(),
            "X-PolyAI-Correlation-Id": correlation_id,
            "Content-Type": "application/json",
        }

        logger.info(f"Making {method} request to {url}")

        # Use requests.request() to handle all HTTP methods uniformly
        api_response = requests.request(
            method=method,
            url=url,
            headers=headers,
            params=params,
            allow_redirects=False,
            data=json.dumps(data) if data else None,
        )

        logger.debug(
            f"Request/response url={url!r} headers={headers!r} body={data!r}"
            f" status_code={api_response.status_code!r} response={api_response.text!r}"
        )

        try:
            api_response.raise_for_status()
        except requests.HTTPError as e:
            logger.exception(
                f"Error in request. url={url!r} headers={headers!r} body={data!r}"
                f" status_code={api_response.status_code!r} response={api_response.text!r}"
            )
            raise e

        try:
            api_response = api_response.json()
        except json.JSONDecodeError as e:
            raise ValueError(f"Failed to parse JSON response: {e}")

        logger.info(f"Request to {url} successful")
        return api_response

    @staticmethod
    def get_accounts(region: str) -> dict[str, str]:
        """Get the accounts for a given region.

        Args:
            region (str): The region name

        Returns:
            dict[str, str]: A dictionary mapping account names to account IDs
        """
        accounts = {}
        accounts_data = PlatformAPIHandler.make_request(region, ACCOUNTS_URL, "GET")

        if not isinstance(accounts_data, list):
            raise ValueError("Expected a list of accounts")

        for account in accounts_data:
            if account.get("active", False) and account.get("id") and account.get("name"):
                accounts[account.get("name")] = account.get("id")

        return accounts

    @staticmethod
    def get_projects(region: str, account_id: str) -> dict[str, str]:
        """Get the projects for a given account.

        Args:
            region (str): The region name
            account_id (str): The account ID

        Returns:
            dict[str, str]: A dictionary mapping project names to project IDs
        """
        projects = {}
        endpoint = PROJECTS_URL.format(account_id=account_id)
        projects_data = PlatformAPIHandler.make_request(region, endpoint, "GET")
        projects_list = projects_data.get("projects", [])

        if not isinstance(projects_list, list):
            raise ValueError("Expected a list of projects")

        for project in projects_list:
            if project.get("id") and project.get("name"):
                projects[project.get("name")] = project.get("id")

        return projects

    @staticmethod
    def get_deployments(region: str, account_id: str, project_id: str) -> dict[str, str]:
        """Get the deployments for a given project.
        Args:
            region (str): The region name
            account_id (str): The account ID
            project_id (str): The project ID
        Returns:
            dict[str, str]: A dictionary mapping deployment versions to deployment IDs
        """
        deployments = {}
        endpoint = DEPLOYMENTS_URL.format(account_id=account_id, project_id=project_id)

        deployments_data = PlatformAPIHandler.make_request(
            region, endpoint, "GET", data=None, params={"client_env": "sandbox"}
        )
        deployments_list = deployments_data.get("deployments", [])

        if not isinstance(deployments_list, list):
            raise ValueError("Expected a list of deployments")

        for deployment in deployments_list:
            if deployment.get("id") and deployment.get("version_hash"):
                deployments[deployment.get("version_hash")[:9]] = deployment.get("id")

        return deployments

    @staticmethod
    def get_active_deployments(
        region: str, account_id: str, project_id: str
    ) -> dict[str, dict[str, str]]:
        """Get the active deployments for a given project.
        Args:
            region (str): The region name
            account_id (str): The account ID
            project_id (str): The project ID
        Returns:
            dict[str, dict[str, str]]: A dictionary mapping environments to deployment info
        """
        endpoint = ACTIVE_DEPLOYMENTS_URL.format(account_id=account_id, project_id=project_id)

        deployments_data = PlatformAPIHandler.make_request(region, endpoint, "GET")

        return deployments_data

    @staticmethod
    def create_chat(
        region: str,
        account_id: str,
        project_id: str,
        environment: str = "sandbox",
        variant_id: ty.Optional[str] = None,
        channel: str = "chat.polyai",
    ) -> dict:
        """Create a new chat conversation.

        Args:
            region: The region name
            account_id: The account ID
            project_id: The project ID
            environment: The environment to chat against (sandbox, pre-release, live)
            variant_id: Optional variant ID (e.g. 'Voice')

        Returns:
            dict: The API response containing the conversation ID
        """
        endpoint = CHAT_URL.format(account_id=account_id, project_id=project_id)
        data = {
            "client_env": environment,
            "channel": channel,
        }
        if variant_id:
            data["variant_id"] = variant_id
        return PlatformAPIHandler.make_request(region, endpoint, "POST", data=data)

    @staticmethod
    def send_chat_message(
        region: str,
        account_id: str,
        project_id: str,
        conversation_id: str,
        text: str,
        environment: str = "sandbox",
    ) -> dict:
        """Send a message to an existing chat conversation.

        Args:
            region: The region name
            account_id: The account ID
            project_id: The project ID
            conversation_id: The conversation ID
            text: The user message text
            environment: The environment (sandbox, pre-release, live)

        Returns:
            dict: The API response containing the assistant's reply
        """
        endpoint = CHAT_CONVERSATION_URL.format(
            account_id=account_id,
            project_id=project_id,
            conversation_id=conversation_id,
        )
        data = {"message": text, "client_env": environment}
        return PlatformAPIHandler.make_request(region, endpoint, "POST", data=data)

    @staticmethod
    def end_chat(
        region: str,
        account_id: str,
        project_id: str,
        conversation_id: str,
        environment: str = "sandbox",
    ) -> dict:
        """End a chat conversation.

        Args:
            region: The region name
            account_id: The account ID
            project_id: The project ID
            conversation_id: The conversation ID
            environment: The environment (sandbox, pre-release, live)

        Returns:
            dict: The API response
        """
        endpoint = CHAT_END_URL.format(
            account_id=account_id,
            project_id=project_id,
            conversation_id=conversation_id,
        )
        return PlatformAPIHandler.make_request(
            region, endpoint, "POST", data={"client_env": environment}
        )

    @staticmethod
    def create_draft_chat(
        region: str,
        account_id: str,
        project_id: str,
        artifact_version: str,
        lambda_deployment_version: str,
        channel: str = "chat.polyai",
        variant_id: ty.Optional[str] = None,
    ) -> dict:
        """Create a new chat conversation against a branch deployment.

        Args:
            region: The region name
            account_id: The account ID
            project_id: The project ID
            artifact_version: Branch artifact version from sourcerer
            lambda_deployment_version: Branch lambda version from sourcerer
            channel: The channel identifier (e.g. 'chat.polyai', 'webchat.polyai')
            variant_id: Optional variant ID (e.g. 'Voice')

        Returns:
            dict: The API response containing the conversation ID
        """
        endpoint = DRAFT_CHAT_URL.format(account_id=account_id, project_id=project_id)
        data = {
            "artifact_version": artifact_version,
            "lambda_deployment_version": lambda_deployment_version,
            "channel": channel,
        }
        if variant_id:
            data["variant_id"] = variant_id
        return PlatformAPIHandler.make_request(region, endpoint, "POST", data=data)

    @staticmethod
    def send_draft_chat_message(
        region: str,
        account_id: str,
        project_id: str,
        conversation_id: str,
        text: str,
    ) -> dict:
        """Send a message to an existing draft chat conversation.

        Args:
            region: The region name
            account_id: The account ID
            project_id: The project ID
            conversation_id: The conversation ID
            text: The user message text

        Returns:
            dict: The API response containing the assistant's reply
        """
        endpoint = DRAFT_CHAT_CONVERSATION_URL.format(
            account_id=account_id,
            project_id=project_id,
            conversation_id=conversation_id,
        )
        data = {"message": text}
        return PlatformAPIHandler.make_request(region, endpoint, "POST", data=data)



"""Variable resource for ADK

Variables are a special resource type that does not exist as files on the
filesystem. Instead they are stored in-memory and are used for referencing in
topics and functions.

Copyright PolyAI Limited
"""

import logging
import os
from dataclasses import dataclass

import poly.resources.resource_utils as utils
from poly.handlers.protobuf.variables_pb2 import (
    Variable_Create,
    Variable_Delete,
    Variable_Update,
)
from poly.resources.function import Function
from poly.resources.resource import Resource, ResourceMapping

logger = logging.getLogger(__name__)


@dataclass
class Variable(Resource):
    """A variable resource.

    Variables do not have a corresponding file on disk.

    They are tracked via resource mapping and are denoted through the vrbl suffix.
    """

    resource_id: str
    name: str
    references: dict | None = None

    def __init__(
        self,
        *,
        resource_id: str,
        name: str,
        references: dict | None = None,
    ):
        self.resource_id = resource_id
        self.name = name
        self.references = references

    @staticmethod
    def get_resource_prefix(**kwargs) -> str:
        return "vrbl"

    @property
    def file_path(self) -> str:
        """File path for the resource."""
        return os.path.join("variables", self.name)

    @classmethod
    def read_local_resource(
        cls, file_path: str, resource_id: str, resource_name: str, **kwargs
    ) -> "Variable":
        return cls(resource_id=resource_id, name=resource_name)

    @property
    def raw(self) -> str:
        return f"vrbl:{self.name}"

    def save(self, base_path: str, **kwargs) -> None:
        """Variables are not saved to disk."""
        pass

    @classmethod
    def read_from_file(cls, file_path: str) -> str:
        file_name = os.path.basename(file_path)
        return f"vrbl:{file_name}"

    @classmethod
    def delete_resource(cls, file_path: str, **kwargs) -> None:
        """Variables have no file to delete."""
        pass

    @staticmethod
    def make_pretty(
        contents: str, resource_mappings: list[ResourceMapping] = None, **kwargs
    ) -> str:
        """Replace resource IDs with resource names in the provided contents."""
        return contents

    @staticmethod
    def from_pretty(
        contents: str, resource_mappings: list[ResourceMapping] = None, **kwargs
    ) -> str:
        """Replace resource names with resource IDs in the provided contents."""
        return contents

    @property
    def command_type(self) -> str:
        return "variable"

    @property
    def delete_command_type(self) -> str:
        return "variable_delete"

    @property
    def create_command_type(self) -> str:
        return "variable_create"

    @property
    def update_command_type(self) -> str:
        return "variable_update"

    def build_create_proto(self) -> Variable_Create:
        return Variable_Create(
            id=self.resource_id,
            name=self.name,
        )

    def build_update_proto(self) -> Variable_Update:
        return Variable_Update(
            id=self.resource_id,
            name=self.name,
            references=self.references,
        )

    def build_delete_proto(self) -> Variable_Delete:
        return Variable_Delete(id=self.resource_id)

    def validate(self, **kwargs) -> None:
        pass

    @staticmethod
    def discover_resources(base_path: str) -> list[str]:
        """Variables are not discovered from the filesystem."""
        # Get all function files in project
        functions_files = []
        global_functions_files = os.path.join(base_path, "functions")
        if os.path.exists(global_functions_files):
            functions_files.extend(
                [
                    os.path.join(global_functions_files, file_name)
                    for file_name in os.listdir(global_functions_files)
                    if file_name.endswith(".py")
                ]
            )
        flows_path = os.path.join(base_path, "flows")
        if os.path.exists(flows_path):
            for flow_name in os.listdir(flows_path):
                flow_functions_files = os.path.join(base_path, "flows", flow_name, "functions")
                if os.path.exists(flow_functions_files):
                    functions_files.extend(
                        [
                            os.path.join(flow_functions_files, file_name)
                            for file_name in os.listdir(flow_functions_files)
                            if file_name.endswith(".py")
                        ]
                    )
                function_step_files = os.path.join(base_path, "flows", flow_name, "function_steps")
                if os.path.exists(function_step_files):
                    functions_files.extend(
                        [
                            os.path.join(function_step_files, file_name)
                            for file_name in os.listdir(function_step_files)
                            if file_name.endswith(".py")
                        ]
                    )

        if not functions_files:
            return []

        variable_paths = set()
        for function_file in functions_files:
            function_code = Function.read_from_file(function_file)
            for variable in utils.extract_variable_names_from_code(function_code):
                variable_paths.add(os.path.join(base_path, "variables", variable))
        return list(variable_paths)
